// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "audio_adpcm_adp.h"
#include "es_audio.h"
#include <string.h>

#define DUMMY_ADPCM_MAGIC 0x1234

typedef ES_S32 DummyAdpcmEncoderHandle;
typedef ES_S32 DummyAdpcmDecoderHandle;

static ES_S32 gEncoder[MAX_ADPCM_CODECS];
static ES_S32 gDecoder[MAX_ADPCM_CODECS];

static ES_S32 OpenDummyAdpcmEncoder(ES_VOID *Attr, ES_VOID **Encoder)
{
    ES_S32 i = 0;

    for (i = 0; i < MAX_ADPCM_CODECS; i++) {
        if (gEncoder[i] == 0) {
            gEncoder[i] = DUMMY_ADPCM_MAGIC;
            *Encoder = &gEncoder[i];
            break;
        }
    }

    if (i == MAX_ADPCM_CODECS) {
        printf("OpenDummyAdpcmEncoder failed\n");
        return ES_FAILURE;
    }

    return ES_SUCCESS;
}

static ES_S32 EncodeDummyAdpcmFrame(ES_VOID *Encoder, const ES_U8 *Input, ES_U32 InputSize, ES_U8 *Output,
                                    ES_U32 *OutSize)
{
    DummyAdpcmEncoderHandle *handle = (DummyAdpcmEncoderHandle *)Encoder;

    if (NULL == handle || *handle != DUMMY_ADPCM_MAGIC) {
        printf("Invalid encoder handle\n");
        return ES_FAILURE;
    }

    if (!Input || !Output || InputSize == 0) {
        printf("Invalid argument\n");
        return ES_FAILURE;
    }

    memcpy(Output, Input, InputSize);
    *OutSize = InputSize;

    return ES_SUCCESS;
}

static ES_S32 CloseDummyAdpcmEncoder(ES_VOID *Encoder)
{
    DummyAdpcmEncoderHandle *handle = (DummyAdpcmEncoderHandle *)Encoder;

    if (NULL == handle || *handle != DUMMY_ADPCM_MAGIC) {
        printf("Invalid Encoder handle\n");
        return ES_FAILURE;
    }

    *handle = 0;

    return ES_SUCCESS;
}

static ES_S32 OpenDummyAdpcmDecoder(ES_VOID *Attr, ES_VOID **Decoder)
{
    ES_S32 i = 0;

    for (i = 0; i < MAX_ADPCM_CODECS; i++) {
        if (gDecoder[i] == 0) {
            gDecoder[i] = DUMMY_ADPCM_MAGIC;
            *Decoder = &gDecoder[i];
            break;
        }
    }

    if (i == MAX_ADPCM_CODECS) {
        printf("OpenDummyAdpcmDecoder failed\n");
        return ES_FAILURE;
    }

    return ES_SUCCESS;
}

static ES_S32 DecodeDummyAdpcmFrame(ES_VOID *Decoder, const ES_U8 *Input, ES_S32 InputSize, ES_U8 *Output,
                                    ES_U32 *OutSize)
{
    DummyAdpcmDecoderHandle *handle = (DummyAdpcmDecoderHandle *)Decoder;

    if (NULL == handle || *handle != DUMMY_ADPCM_MAGIC) {
        printf("Invalid Decoder handle\n");
        return ES_FAILURE;
    }

    if (!Input || !Output || InputSize == 0) {
        printf("Invalid argument\n");
        return ES_FAILURE;
    }

    memcpy(Output, Input, InputSize);
    *OutSize = InputSize;

    return ES_SUCCESS;
}

static ES_S32 GetDummyAdpcmStreamInfo(ES_VOID *Decoder, ES_VOID *Info)
{
    DummyAdpcmDecoderHandle *handle = (DummyAdpcmDecoderHandle *)Decoder;
    ADEC_FRAME_INFO *streamInfo = (ADEC_FRAME_INFO *)Info;

    if (NULL == handle || *handle != DUMMY_ADPCM_MAGIC) {
        printf("Invalid Decoder handle\n");
        return ES_FAILURE;
    }

    if (!streamInfo || !streamInfo->Input || streamInfo->InputSize <= 0) {
        printf("Invalid argument\n");
        return ES_FAILURE;
    }

    streamInfo->BitDepth = 16;
    streamInfo->Channels = 2;
    streamInfo->FrameOffset = 0;
    streamInfo->FrameSize = ADPCM_SAMPLES_PER_FRAME;
    streamInfo->SampleRate = 16000;

    return ES_SUCCESS;
}

static ES_S32 ResetDummyAdpcmDecoder(ES_VOID *Decoder) { return ES_SUCCESS; }

static ES_S32 CloseDummyAdpcmDecoder(ES_VOID *Decoder)
{
    DummyAdpcmDecoderHandle *handle = (DummyAdpcmDecoderHandle *)Decoder;

    if (NULL == handle || *handle != DUMMY_ADPCM_MAGIC) {
        printf("Invalid Decoder handle\n");
        return ES_FAILURE;
    }

    *handle = 0;

    return ES_SUCCESS;
}

ES_S32 ES_AENC_AdpcmInit(ES_S32 *handle)
{
    AENC_ENCODER_S adpcm;

    adpcm.Type = PT_ADPCMA;
    snprintf(adpcm.Name, sizeof(adpcm.Name), "Adpcm");
    adpcm.MaxFrmLen = MAX_ADPCM_SAMPLES_PER_FRAME;
    adpcm.FunOpenEncoder = OpenDummyAdpcmEncoder;
    adpcm.FunEncodeFrm = EncodeDummyAdpcmFrame;
    adpcm.FunCloseEncoder = CloseDummyAdpcmEncoder;

    return ES_AENC_RegisterEncoder(handle, &adpcm);
}

ES_S32 ES_ADEC_AdpcmInit(ES_S32 *handle)
{
    ADEC_DECODER_S adpcm;

    adpcm.Type = PT_ADPCMA;
    snprintf(adpcm.Name, sizeof(adpcm.Name), "Adpcm");
    adpcm.FunOpenDecoder = OpenDummyAdpcmDecoder;
    adpcm.FunGetFrmInfo = GetDummyAdpcmStreamInfo;
    adpcm.FunDecodeFrm = DecodeDummyAdpcmFrame;
    adpcm.FunCloseDecoder = CloseDummyAdpcmDecoder;
    adpcm.FunResetDecoder = ResetDummyAdpcmDecoder;

    return ES_ADEC_RegisterDecoder(handle, &adpcm);
}

ES_S32 ES_AENC_AdpcmDeInit(ES_S32 handle) { return ES_AENC_UnRegisterEncoder(handle); }

ES_S32 ES_ADEC_AdpcmDeInit(ES_S32 handle) { return ES_ADEC_UnRegisterDecoder(handle); }
