// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <pthread.h>
#include <errno.h>
#include <signal.h>
#include <dirent.h>
#include "sample_comm.h"
#include "es_audio.h"
#include "audio_adpcm_adp.h"
#include "utils.h"
#include "es-numa.h"

#define AACLC_SAMPLES_PER_FRAME 1024
#define AMRNB_SAMPLES_PER_FRAME 160
#define AMRWB_SAMPLES_PER_FRAME 320
#define INVALID_AUDIO_CARD -1
#define INVALID_AUDIO_DEV -1
#define MULTI_AO_MAX_NUM 6
#define SAMPLE_DBG(s32Ret)                                                     \
    do {                                                                       \
        printf("s32Ret=%#x,fuc:%s,line:%d\n", s32Ret, __FUNCTION__, __LINE__); \
    } while (0)

static ES_BOOL gs_bUserGetMode = ES_FALSE;
static ES_BOOL gs_bAioReSample = ES_FALSE;
static ES_BOOL gs_bAoVolumeCtrl = ES_FALSE;
static AUDIO_SAMPLE_RATE_E enInSampleRate = AUDIO_SAMPLE_RATE_BUTT;
static AUDIO_SAMPLE_RATE_E enOutSampleRate = AUDIO_SAMPLE_RATE_BUTT;
static ES_U32 u32AiVqeType = 1;

#ifndef __ADPCM_DEC_SUPPORT__
static PAYLOAD_TYPE_E gs_enPayloadType = PT_AAC;
#else
static PAYLOAD_TYPE_E gs_enPayloadType = PT_ADPCMA;
#endif

ES_S32 get_device(ES_CHAR* content, AUDIO_CARD* card, AUDIO_DEV* device)
{
    if (content != NULL) {
        if (sscanf(content, "%d,%d", card, device) != 2) {
            printf("Invalid device format. Expected format: <card>,<device>\n");
            return ES_FAILURE;
        }
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : PT Number to String
 ******************************************************************************/
static char* SAMPLE_AUDIO_Pt2Str(PAYLOAD_TYPE_E enType)
{
    if (PT_G711A == enType) {
        return "g711a";
    } else if (PT_G711U == enType) {
        return "g711u";
    } else if (PT_G722 == enType) {
        return "g722";
    } else if (PT_G726 == enType) {
        return "g726";
    } else if (PT_AAC == enType) {
        return "aac";
    } else if (PT_MP3 == enType) {
        return "mp3";
    } else if (PT_AMR == enType) {
        return "amr";
    } else if (PT_ADPCMA == enType) {
        return "adpcm";
    } else {
        return "data";
    }
}

/******************************************************************************
 * function : Open Aenc File
 ******************************************************************************/
static FILE* SAMPLE_AUDIO_OpenAencFile(AENC_CHN AeChn, PAYLOAD_TYPE_E enType)
{
    FILE* pfd;
    ES_CHAR aszFileName[FILE_NAME_LEN] = {0};

    snprintf(aszFileName, FILE_NAME_LEN, "/tmp/aenc_chn%d.%s", AeChn, SAMPLE_AUDIO_Pt2Str(enType));
    pfd = fopen(aszFileName, "w+");
    if (NULL == pfd) {
        printf("%s: open file %s failed\n", __FUNCTION__, aszFileName);
        return NULL;
    }
    printf("open stream file:\"%s\" for aenc ok\n", aszFileName);

    return pfd;
}

/******************************************************************************
 * function : Open Adec File
 ******************************************************************************/
static FILE* SAMPLE_AUDIO_OpenAdecFile(ADEC_CHN AdChn, PAYLOAD_TYPE_E enType)
{
    FILE* pfd;
    ES_CHAR aszFileName[FILE_NAME_LEN] = {0};

    snprintf(aszFileName, FILE_NAME_LEN, "/opt/eswin/data/audio/adec_chn%d.%s", AdChn, SAMPLE_AUDIO_Pt2Str(enType));
    pfd = fopen(aszFileName, "rb");
    if (NULL == pfd) {
        printf("%s: open file %s failed\n", __FUNCTION__, aszFileName);
        return NULL;
    }
    printf("open stream file:\"%s\" for adec ok\n", aszFileName);

    return pfd;
}

/******************************************************************************
 * function : Open Adec output File
 ******************************************************************************/
static FILE* SAMPLE_AUDIO_OpenAdecOutputFile(ADEC_CHN AdChn, PAYLOAD_TYPE_E enType)
{
    FILE* pfd;
    ES_CHAR aszFileName[FILE_NAME_LEN] = {0};

    snprintf(aszFileName, FILE_NAME_LEN, "/tmp/adec_chn%d.%s.raw", AdChn, SAMPLE_AUDIO_Pt2Str(enType));
    pfd = fopen(aszFileName, "wb");
    if (NULL == pfd) {
        printf("%s: open file %s failed\n", __FUNCTION__, aszFileName);
        return NULL;
    }
    printf("open stream file:\"%s\" for adec ok\n", aszFileName);

    return pfd;
}

/******************************************************************************
 * function : Open AI File
 ******************************************************************************/
static FILE* SAMPLE_AUDIO_OpenAiFile(AUDIO_CARD card, AUDIO_DEV dev)
{
    FILE* pfd;
    ES_CHAR aszFileName[FILE_NAME_LEN] = {0};

    snprintf(aszFileName, FILE_NAME_LEN, "/tmp/ai_card%d.dev%d.raw", card, dev);
    pfd = fopen(aszFileName, "wb");
    if (NULL == pfd) {
        printf("%s: open file %s failed\n", __FUNCTION__, aszFileName);
        return NULL;
    }
    printf("open stream file:\"%s\" for AI ok\n", aszFileName);

    return pfd;
}

/******************************************************************************
 * function : Open AO File
 ******************************************************************************/
static FILE* SAMPLE_AUDIO_OpenAoFile()
{
    FILE* pfd;
    ES_CHAR aszFileName[FILE_NAME_LEN] = {0};

    snprintf(aszFileName, FILE_NAME_LEN, "/opt/eswin/data/audio/ao.wav");
    pfd = fopen(aszFileName, "rb");
    if (NULL == pfd) {
        printf("%s: open file %s failed\n", __FUNCTION__, aszFileName);
        return NULL;
    }
    printf("open stream file:\"%s\" for AO ok\n", aszFileName);

    return pfd;
}

/******************************************************************************
 * function : Open Aenc Wav File
 ******************************************************************************/
static FILE* SAMPLE_AUDIO_OpenAencWavFile()
{
    FILE* pfd;
    ES_CHAR aszFileName[FILE_NAME_LEN] = {0};

    snprintf(aszFileName, FILE_NAME_LEN, "/opt/eswin/data/audio/aenc.wav");
    pfd = fopen(aszFileName, "rb");
    if (NULL == pfd) {
        printf("%s: open file %s failed\n", __FUNCTION__, aszFileName);
        return NULL;
    }
    printf("open stream file:\"%s\" for AENC ok\n", aszFileName);

    return pfd;
}

/******************************************************************************
 * function : AI
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_AI(AUDIO_CARD AiCard, AUDIO_DEV AiDev)
{
    ES_S32 s32Ret;
    FILE* pfd;
    AIO_ATTR_S stAioAttr;
    ES_BOOL bSaveFile, bTrackMode;
    AUDIO_SAVE_FILE_INFO_S SaveFileInfo;
    AUDIO_TRACK_MODE_E enTrackMode;
    ES_BOOL bAiCtrlVol;

    stAioAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
    stAioAttr.Bitwidth = ES_AI_HW_BIT_WIDTH;
    stAioAttr.HwBitwidth = ES_AI_HW_BIT_WIDTH;
    stAioAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
    stAioAttr.ChnCnt = 2;
    stAioAttr.PtNumPerFrm = SAMPLE_AUDIO_PTNUMPERFRM;
    stAioAttr.FrmNum = 30;

    bSaveFile = ES_TRUE;
    SaveFileInfo.Cfg = ES_TRUE;
    snprintf(SaveFileInfo.FilePath, MAX_AUDIO_FILE_PATH_LEN, "./");
    snprintf(SaveFileInfo.FileName, MAX_AUDIO_FILE_NAME_LEN, "ai_save");
    SaveFileInfo.FileSize = 4096;

    bTrackMode = ES_TRUE;
    enTrackMode = AUDIO_TRACK_NORMAL;

    bAiCtrlVol = ES_TRUE;

    s32Ret = SAMPLE_COMM_AUDIO_StartAi(AiCard, AiDev, &stAioAttr, AUDIO_SAMPLE_RATE_BUTT, ES_FALSE, NULL, 0);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto AI_START_FAILED;
    }

    s32Ret = SAMPLE_COMM_AUDIO_AiSetting(AiCard, AiDev, &SaveFileInfo, bSaveFile, enTrackMode, bTrackMode);
    if (ES_SUCCESS != s32Ret) {
        SAMPLE_DBG(s32Ret);
    }

    pfd = SAMPLE_AUDIO_OpenAiFile(AiCard, AiDev);
    if (!pfd) {
        SAMPLE_DBG(ES_FAILURE);
        goto AI_OPEN_FILE_FAILED;
    }

    s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAi(AiCard, AiDev, pfd);
    if (ES_SUCCESS != s32Ret) {
        SAMPLE_DBG(s32Ret);
        goto AI_CREATE_TRD_FAILED1;
    }

    if (ES_TRUE == bAiCtrlVol) {
        s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAiVolCtrl(AiCard, AiDev);
        if (ES_SUCCESS != s32Ret) {
            SAMPLE_DBG(s32Ret);
            goto AI_CREATE_TRD_FAILED0;
        }
    }

    printf("AI(%d, %d) start success", AiCard, AiDev);
    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

    if (ES_TRUE == bAiCtrlVol) {
        s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAiVolCtrl(AiCard, AiDev);
    }

AI_CREATE_TRD_FAILED0:
    s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAi(AiCard, AiDev);
AI_CREATE_TRD_FAILED1:
    fclose(pfd);
AI_OPEN_FILE_FAILED:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAi(AiCard, AiDev, ES_FALSE, ES_FALSE);
AI_START_FAILED:
    return s32Ret;
}

/******************************************************************************
 * function : AO
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_AO(AUDIO_CARD AoCard, AUDIO_DEV AoDev)
{
    ES_S32 s32Ret = ES_FAILURE;
    FILE* pfd;
    AIO_ATTR_S stAioAttr;
    ES_BOOL bSaveFile, bTrackMode;
    AUDIO_SAVE_FILE_INFO_S SaveFileInfo;
    AUDIO_TRACK_MODE_E enTrackMode;
    ES_BOOL bAoCtrlVol;
    ES_BOOL bAoCtrlPause;

    stAioAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
    stAioAttr.Bitwidth = AUDIO_BIT_WIDTH_32;
    stAioAttr.HwBitwidth = AUDIO_BIT_WIDTH_32;
    stAioAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
    stAioAttr.ChnCnt = 2;
    stAioAttr.PtNumPerFrm = 512;
    stAioAttr.FrmNum = 4;

    bSaveFile = ES_TRUE;
    SaveFileInfo.Cfg = ES_TRUE;
    snprintf(SaveFileInfo.FilePath, MAX_AUDIO_FILE_PATH_LEN, "./");
    snprintf(SaveFileInfo.FileName, MAX_AUDIO_FILE_NAME_LEN, "ao_save");
    SaveFileInfo.FileSize = 4096;

    bTrackMode = ES_TRUE;
    enTrackMode = AUDIO_TRACK_NORMAL;

    bAoCtrlVol = ES_FALSE;
    bAoCtrlPause = ES_FALSE;

    pfd = SAMPLE_AUDIO_OpenAoFile();
    if (!pfd) {
        SAMPLE_DBG(ES_FAILURE);
        goto AO_OPEN_FILE_FAILED;
    }

    struct chunk_fmt stChunkFmt;
    s32Ret = parseWav(pfd, &stChunkFmt);
    if (ES_SUCCESS != s32Ret) {
        SAMPLE_DBG(ES_FAILURE);
        goto AO_PARSE_WAV_FAILED;
    }

    stAioAttr.Bitwidth = convertBitsToBitWidth(stChunkFmt.bits_per_sample);
    stAioAttr.Soundmode = convertChnsToSoundMode(stChunkFmt.num_channels);
    stAioAttr.ChnCnt = stChunkFmt.num_channels;
    stAioAttr.Samplerate = (AUDIO_SAMPLE_RATE_E)stChunkFmt.sample_rate;

    s32Ret = SAMPLE_COMM_AUDIO_StartAo(AoCard, AoDev, stAioAttr.ChnCnt, &stAioAttr, stAioAttr.Samplerate, ES_FALSE);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto AO_START_FAILED;
    }

    s32Ret = SAMPLE_COMM_AUDIO_AoSetting(AoCard, AoDev, &SaveFileInfo, bSaveFile, enTrackMode, bTrackMode);
    if (ES_SUCCESS != s32Ret) {
        SAMPLE_DBG(s32Ret);
        goto AO_SETTING_FAILED;
    }

    s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAo(AoCard, AoDev, &stAioAttr, pfd);
    if (ES_SUCCESS != s32Ret) {
        SAMPLE_DBG(s32Ret);
        goto AO_CREATE_TRD_FAILED1;
    }

    if (ES_TRUE == bAoCtrlVol) {
        s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAoVolCtrl(AoCard, AoDev);
        if (ES_SUCCESS != s32Ret) {
            SAMPLE_DBG(s32Ret);
            goto AO_CREATE_TRD_Vol_FAILED;
        }
    }

    if (ES_TRUE == bAoCtrlPause) {
        s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAoPauseCtrl(AoCard, AoDev);
        if (ES_SUCCESS != s32Ret) {
            SAMPLE_DBG(s32Ret);
            goto AO_CREATE_TRD_Pause_FAILED;
        }
    }

    printf("AO(%d, %d) start success\n", AoCard, AoDev);
    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

    if (ES_TRUE == bAoCtrlPause) {
        s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAoPauseCtrl(AoCard, AoDev);
    }

AO_CREATE_TRD_Pause_FAILED:
    if (ES_TRUE == bAoCtrlVol) {
        s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAoVolCtrl(AoCard, AoDev);
    }
AO_CREATE_TRD_Vol_FAILED:
    s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAo(AoCard, AoDev);
AO_CREATE_TRD_FAILED1:
AO_SETTING_FAILED:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAo(AoCard, AoDev, stAioAttr.ChnCnt >> stAioAttr.Soundmode, ES_FALSE);
AO_START_FAILED:
AO_PARSE_WAV_FAILED:
    fclose(pfd);
AO_OPEN_FILE_FAILED:
    return s32Ret;
}
typedef struct {
    int card;
    int device;
} PlaybackNode;

int get_playback_nodes(PlaybackNode* nodes, int max_nodes)
{
    DIR* dir;
    struct dirent* entry;
    int playback_count = 0;

    dir = opendir("/dev/snd");
    if (dir == NULL) {
        perror("opendir");
        return -1;
    }

    while ((entry = readdir(dir)) != NULL) {
        if (strncmp(entry->d_name, "pcmC", 4) == 0 && entry->d_name[strlen(entry->d_name) - 1] == 'p') {
            int card, device;

            if (sscanf(entry->d_name, "pcmC%dD%dp", &card, &device) == 2) {
                if (playback_count < max_nodes) {
                    nodes[playback_count].card = card;
                    nodes[playback_count].device = device;
                }
                playback_count++;
            }
        }
    }

    closedir(dir);

    return playback_count;
}

/******************************************************************************
 * function : AO dual-DIE sample
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_DUAL_DIE_AO(ES_VOID)
{
    ES_S32 s32Ret = ES_FAILURE;
    ES_S32 index = 0;

    ES_S32 numaAvail = ES_NUMA_Avaiable();
    if (!numaAvail) {
        printf("Only DIE 0 is supported on this hardware. Please run this case on dual-DIE hardware.\n");
        return ES_FAILURE;
    }

    PlaybackNode nodes[MULTI_AO_MAX_NUM];
    int total_nodes = get_playback_nodes(nodes, MULTI_AO_MAX_NUM);

    if (total_nodes < 0) {
        fprintf(stderr, "Error reading playback nodes\n");
        return EXIT_FAILURE;
    }

    printf("Total number of playback nodes: %d\n", total_nodes);

    FILE* pfd[MULTI_AO_MAX_NUM] = {NULL};
    AIO_ATTR_S stAioAttr;
    stAioAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
    stAioAttr.Bitwidth = AUDIO_BIT_WIDTH_32;
    stAioAttr.HwBitwidth = AUDIO_BIT_WIDTH_32;
    stAioAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
    stAioAttr.ChnCnt = 2;
    stAioAttr.PtNumPerFrm = 512;
    stAioAttr.FrmNum = 4;

    for (index = 0; index < total_nodes; index++) {
        pfd[index] = SAMPLE_AUDIO_OpenAoFile();
        if (!pfd[index]) {
            SAMPLE_DBG(ES_FAILURE);
            goto AO_OPEN_FILE_FAILED;
        }

        struct chunk_fmt stChunkFmt;
        s32Ret = parseWav(pfd[index], &stChunkFmt);
        if (ES_SUCCESS != s32Ret) {
            SAMPLE_DBG(ES_FAILURE);
            goto AO_PARSE_WAV_FAILED;
        }

        stAioAttr.Bitwidth = convertBitsToBitWidth(stChunkFmt.bits_per_sample);
        stAioAttr.Soundmode = convertChnsToSoundMode(stChunkFmt.num_channels);
        stAioAttr.ChnCnt = stChunkFmt.num_channels;
        stAioAttr.Samplerate = (AUDIO_SAMPLE_RATE_E)stChunkFmt.sample_rate;

        s32Ret = SAMPLE_COMM_AUDIO_StartAo(nodes[index].card, nodes[index].device, stAioAttr.ChnCnt, &stAioAttr,
                                           stAioAttr.Samplerate, ES_FALSE);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AO_START_FAILED;
        }

        s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAo(nodes[index].card, nodes[index].device, &stAioAttr, pfd[index]);
        if (ES_SUCCESS != s32Ret) {
            SAMPLE_DBG(s32Ret);
            goto AO_CREATE_TRD_FAILED;
        }

        printf("AO(%d, %d) start success\n", nodes[index].card, nodes[index].device);
    }

    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

    for (index = 0; index < total_nodes; index++) {
        s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAo(nodes[index].card, nodes[index].device);
    }

AO_CREATE_TRD_FAILED:
    for (index = 0; index < total_nodes; index++) {
        s32Ret |= SAMPLE_COMM_AUDIO_StopAo(nodes[index].card, nodes[index].device,
                                           stAioAttr.ChnCnt >> stAioAttr.Soundmode, ES_FALSE);
    }
AO_START_FAILED:
AO_PARSE_WAV_FAILED:
    for (index = 0; index < total_nodes; index++) {
        if (!pfd[index]) {
            fclose(pfd[index]);
        }
    }
AO_OPEN_FILE_FAILED:
    return s32Ret;
}

/******************************************************************************
 * function : AENC
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_AENC(ES_VOID)
{
    ES_S32 s32Ret = ES_FAILURE;
    AENC_CHN AencChn = 0;
    AIO_ATTR_S stAioAttr;
    struct chunk_fmt stChunkFmt;
    FILE *pInFd = NULL, *pOutFd = NULL;

    pInFd = SAMPLE_AUDIO_OpenAencWavFile(0, 0);
    if (NULL == pInFd) {
        SAMPLE_DBG(ES_FAILURE);
        goto AENC_OPEN_INFILE_FAILED;
    }

    s32Ret = parseWav(pInFd, &stChunkFmt);
    if (ES_SUCCESS != s32Ret) {
        SAMPLE_DBG(ES_FAILURE);
        goto AENC_PARSE_WAV_FAILED;
    }

    stAioAttr.Bitwidth = convertBitsToBitWidth(stChunkFmt.bits_per_sample);
    stAioAttr.Soundmode = convertChnsToSoundMode(stChunkFmt.num_channels);
    stAioAttr.ChnCnt = stChunkFmt.num_channels;
    stAioAttr.Samplerate = (AUDIO_SAMPLE_RATE_E)stChunkFmt.sample_rate;
    stAioAttr.FrmNum = 30;
#ifndef ____ADPCM_DEC_SUPPORT__
    stAioAttr.PtNumPerFrm = AACLC_SAMPLES_PER_FRAME;
#else
    stAioAttr.PtNumPerFrm = ADPCM_SAMPLES_PER_FRAME;
#endif

    s32Ret = SAMPLE_COMM_AUDIO_StartAenc(AencChn, &stAioAttr, gs_enPayloadType);
    if (ES_SUCCESS != s32Ret) {
        SAMPLE_DBG(s32Ret);
        goto AENC_PARSE_WAV_FAILED;
    }

    pOutFd = SAMPLE_AUDIO_OpenAencFile(AencChn, gs_enPayloadType);
    if (NULL == pOutFd) {
        SAMPLE_DBG(s32Ret);
        goto AENC_OPEN_OUTFILE_FAILED;
    }

    s32Ret = SAMPLE_COMM_AUDIO_CreateTrdAenc(AencChn, &stAioAttr, pInFd, pOutFd);
    if (ES_SUCCESS != s32Ret) {
        SAMPLE_DBG(s32Ret);
        goto AENC_CREATE_TRD_FAILED;
    }

    printf("aenc(%d) start ok\n", AencChn);
    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

    s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAenc(AencChn);

AENC_CREATE_TRD_FAILED:
    fclose(pOutFd);
AENC_OPEN_OUTFILE_FAILED:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAenc(AencChn);
AENC_PARSE_WAV_FAILED:
    fclose(pInFd);
AENC_OPEN_INFILE_FAILED:
    return s32Ret;
}

/******************************************************************************
 * function : ADEC
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_ADEC(ES_VOID)
{
    ES_S32 s32Ret = ES_FAILURE;
    ADEC_CHN AdecChn = 0;
    FILE *pInFd = NULL, *pOutFd = NULL;

    pInFd = SAMPLE_AUDIO_OpenAdecFile(AdecChn, gs_enPayloadType);
    if (!pInFd) {
        SAMPLE_DBG(ES_FAILURE);
        goto ADEC_OpenAdecFile_FAILED;
    }

    pOutFd = SAMPLE_AUDIO_OpenAdecOutputFile(AdecChn, gs_enPayloadType);
    if (!pOutFd) {
        SAMPLE_DBG(ES_FAILURE);
        goto ADEC_OpenAdecOutputFile_FAILED;
    }

    s32Ret = SAMPLE_COMM_AUDIO_StartAdec(AdecChn, gs_enPayloadType);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto ADEC_StartAdec_FAILED;
    }

    s32Ret = SAMPLE_COMM_AUDIO_CreateTrdAdecGetData(AdecChn, pInFd, pOutFd, INVALID_AUDIO_CARD, INVALID_AUDIO_DEV);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto ADEC_CreateTrdAdecGetData_FAILED;
    }

    s32Ret = SAMPLE_COMM_AUDIO_CreateTrdAdec(AdecChn, pInFd, pOutFd);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto ADEC_CreatTrdFileAdec_FAILED;
    }

    printf("adec(%d) start ok\n", AdecChn);
    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

    s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAdec(AdecChn);

ADEC_CreatTrdFileAdec_FAILED:
    s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAdecGetData(AdecChn);
ADEC_CreateTrdAdecGetData_FAILED:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAdec(AdecChn);
ADEC_StartAdec_FAILED:
    fclose(pOutFd);
ADEC_OpenAdecOutputFile_FAILED:
    fclose(pInFd);
ADEC_OpenAdecFile_FAILED:
    return s32Ret;
}

/******************************************************************************
 * function : file -> Adec -> Ao
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_AdecAo(AUDIO_CARD AoCard, AUDIO_DEV AoDev)
{
    ES_S32 s32Ret;
    ADEC_CHN AdChn = 0;
    ES_S32 s32AoChnCnt;
    FILE* pfd = NULL;
    FILE* pOutFd = NULL;
    AIO_ATTR_S stAioAttr;
    stAioAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
    stAioAttr.Bitwidth = AUDIO_BIT_WIDTH_16;
    stAioAttr.HwBitwidth = ES_AO_HW_BIT_WIDTH;
    stAioAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
    stAioAttr.FrmNum = 4;
    stAioAttr.PtNumPerFrm = 2048;
    stAioAttr.ChnCnt = 2;

    gs_bAioReSample = ES_FALSE;
    enInSampleRate = AUDIO_SAMPLE_RATE_BUTT;
    enOutSampleRate = AUDIO_SAMPLE_RATE_BUTT;

    s32Ret = SAMPLE_COMM_AUDIO_StartAdec(AdChn, gs_enPayloadType);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto ADECAO_StartAdec_EXIT;
    }

    s32AoChnCnt = stAioAttr.ChnCnt;
    s32Ret = SAMPLE_COMM_AUDIO_StartAo(AoCard, AoDev, s32AoChnCnt, &stAioAttr, enInSampleRate, gs_bAioReSample);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto ADECAO_StartAo_EXIT;
    }

    AUDIO_SAVE_FILE_INFO_S SaveFileInfo;
    SaveFileInfo.Cfg = ES_TRUE;
    snprintf(SaveFileInfo.FilePath, MAX_AUDIO_FILE_PATH_LEN, "./");
    snprintf(SaveFileInfo.FileName, MAX_AUDIO_FILE_NAME_LEN, "ao_save");
    SaveFileInfo.FileSize = 4096;
    s32Ret = ES_AO_SaveFile(AoCard, AoDev, &SaveFileInfo);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AI_SaveFile(%d,%d), failed with %#x!\n", __FUNCTION__, AoCard, AoDev, s32Ret);
    }

    pfd = SAMPLE_AUDIO_OpenAdecFile(AdChn, gs_enPayloadType);
    if (!pfd) {
        SAMPLE_DBG(ES_FAILURE);
        goto ADECAO_OpenAdecFile_EXIT;
    }

    pOutFd = SAMPLE_AUDIO_OpenAdecOutputFile(AdChn, gs_enPayloadType);
    if (!pOutFd) {
        SAMPLE_DBG(ES_FAILURE);
        goto ADECAO_OpenAdecOutputFile_EXIT;
    }

    if (ES_TRUE == gs_bUserGetMode) {
        s32Ret = SAMPLE_COMM_AUDIO_CreateTrdAdecGetData(AdChn, pfd, pOutFd, AoCard, AoDev);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto ADECAO_CreateTrdAdecGetData_EXIT;
        }
    } else {
        s32Ret = SAMPLE_COMM_AUDIO_AoBindAdec(AoCard, AoDev, AdChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto ADECAO_AoBindAdec_EXIT;
        }
    }

    s32Ret = SAMPLE_COMM_AUDIO_CreatTrdFileAdec(AdChn, pfd);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto ADECAO_CreatTrdFileAdec_EXIT;
    }

    printf("bind adec:%d to ao(%d,%d) ok \n", AdChn, AoCard, AoDev);
    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

    s32Ret = SAMPLE_COMM_AUDIO_DestroyTrdFileAdec(AdChn);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
    }

ADECAO_CreatTrdFileAdec_EXIT:
    if (ES_TRUE == gs_bUserGetMode) {
        s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAdecGetData(AdChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
        }
    } else {
        s32Ret |= SAMPLE_COMM_AUDIO_AoUnbindAdec(AoCard, AoDev, AdChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
        }
    }
ADECAO_CreateTrdAdecGetData_EXIT:
ADECAO_AoBindAdec_EXIT:
    if (pOutFd) {
        fclose(pOutFd);
        pOutFd = NULL;
    }
ADECAO_OpenAdecOutputFile_EXIT:
    if (pfd) {
        fclose(pfd);
        pfd = NULL;
    }
ADECAO_OpenAdecFile_EXIT:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAo(AoCard, AoDev, s32AoChnCnt >> stAioAttr.Soundmode, gs_bAioReSample);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
    }
ADECAO_StartAo_EXIT:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAdec(AdChn);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
    }
ADECAO_StartAdec_EXIT:
    return s32Ret;
}

/******************************************************************************
 * function : Ai -> Aenc -> file
 *                       -> Adec -> Ao
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_AiAenc(AUDIO_CARD AiCard, AUDIO_DEV AiDev, AUDIO_CARD AoCard, AUDIO_DEV AoDev)
{
    ES_S32 s32Ret;
    ADEC_CHN AdChn = 1;
    ES_S32 s32AoChnCnt;
    AENC_CHN AeChn = 0;
    ES_BOOL bSendAdec = ES_TRUE;
    FILE* pfd = NULL;
    AIO_ATTR_S stAioAttr;
    stAioAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
    stAioAttr.Bitwidth = AUDIO_BIT_WIDTH_16;
    stAioAttr.HwBitwidth = ES_AI_HW_BIT_WIDTH;
    stAioAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
    stAioAttr.FrmNum = 4;
#ifndef __ADPCM_DEC_SUPPORT__
    stAioAttr.PtNumPerFrm = AACLC_SAMPLES_PER_FRAME;
#else
    stAioAttr.PtNumPerFrm = ADPCM_SAMPLES_PER_FRAME;
#endif
    stAioAttr.ChnCnt = 2;

    gs_bAioReSample = ES_FALSE;
    enInSampleRate = AUDIO_SAMPLE_RATE_BUTT;
    enOutSampleRate = AUDIO_SAMPLE_RATE_BUTT;

    /********************************************
      step 1: start Ai
    ********************************************/
    s32Ret = SAMPLE_COMM_AUDIO_StartAi(AiCard, AiDev, &stAioAttr, enOutSampleRate, gs_bAioReSample, NULL, 0);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto AIAENC_START_AI_FAILED;
    }

    /********************************************
      step 2: start Aenc
    ********************************************/
    s32Ret = SAMPLE_COMM_AUDIO_StartAenc(AeChn, &stAioAttr, gs_enPayloadType);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto AIAENC_START_AENC_FAILED;
    }

    /********************************************
      step 3: Aenc bind Ai Chn
    ********************************************/

    if (ES_TRUE == gs_bUserGetMode) {
        s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAiAenc(AiCard, AiDev, AeChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AIAENC_BIND_FAILED;
        }
    } else {
        s32Ret = SAMPLE_COMM_AUDIO_AencBindAi(AiCard, AiDev, AeChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AIAENC_BIND_FAILED;
        }
    }
    printf("Ai(%d,%d) bind to AencChn:%d ok!\n", AiCard, AiDev, AeChn);

    /********************************************
      step 4: start Adec & Ao. ( if you want )
    ********************************************/
    pfd = SAMPLE_AUDIO_OpenAencFile(AeChn, gs_enPayloadType);
    if (!pfd) {
        SAMPLE_DBG(ES_FAILURE);
        goto AIAENC_OPEN_FILE_FAILED;
    }

    if (ES_TRUE == bSendAdec) {
        s32Ret = SAMPLE_COMM_AUDIO_StartAdec(AdChn, gs_enPayloadType);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AIAENC_COMM_FAILED;
        }

        s32AoChnCnt = stAioAttr.ChnCnt;
        AIO_ATTR_S aoAttr = stAioAttr;
        aoAttr.HwBitwidth = ES_AO_HW_BIT_WIDTH;
        s32Ret = SAMPLE_COMM_AUDIO_StartAo(AoCard, AoDev, s32AoChnCnt, &aoAttr, enInSampleRate, gs_bAioReSample);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AIAENC_AO_START_FAILED;
        }

        s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAencAdec(AeChn, AdChn, pfd);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AIAENC_CREAT_AENCAD_FAILED;
        }

        s32Ret = SAMPLE_COMM_AUDIO_AoBindAdec(AoCard, AoDev, AdChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AIAENC_AO_BIND_ADEC_FAILED;
        }

        printf("bind adec:%d to ao(%d,%d) ok \n", AdChn, AoCard, AoDev);
    } else {
        s32Ret = SAMPLE_COMM_AUDIO_CreateTrdAenc(AeChn, NULL, NULL, pfd);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AIAENC_COMM_FAILED;
        }
    }

    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

    /********************************************
      step 5: exit the process
    ********************************************/
    if (ES_TRUE == bSendAdec) {
        s32Ret = SAMPLE_COMM_AUDIO_AoUnbindAdec(AoCard, AoDev, AdChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            return ES_FAILURE;
        }

AIAENC_AO_BIND_ADEC_FAILED:
        s32Ret = SAMPLE_COMM_AUDIO_DestroyTrdAencAdec(AeChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
        }
AIAENC_CREAT_AENCAD_FAILED:
        s32Ret |= SAMPLE_COMM_AUDIO_StopAo(AoCard, AoDev, s32AoChnCnt >> stAioAttr.Soundmode, gs_bAioReSample);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
        }
AIAENC_AO_START_FAILED:
        s32Ret |= SAMPLE_COMM_AUDIO_StopAdec(AdChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
        }
    } else {
        s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAenc(AeChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
        }
    }

AIAENC_COMM_FAILED:
    fclose(pfd);
AIAENC_OPEN_FILE_FAILED:
    if (ES_TRUE == gs_bUserGetMode) {
        s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAi(AiCard, AiDev);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
        }
    } else {
        s32Ret |= SAMPLE_COMM_AUDIO_AencUnbindAi(AiCard, AiDev, AeChn);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
        }
    }
AIAENC_BIND_FAILED:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAenc(AeChn);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
    }
AIAENC_START_AENC_FAILED:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAi(AiCard, AiDev, gs_bAioReSample, ES_FALSE);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
    }
AIAENC_START_AI_FAILED:
    return s32Ret;
}

/******************************************************************************
 * function : Ai -> Ao(with fade in/out and volume adjust)
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_AiAo(AUDIO_CARD AiCard, AUDIO_DEV AiDev, AUDIO_CARD AoCard, AUDIO_DEV AoDev)
{
    ES_S32 s32Ret;
    ES_S32 s32AoChnCnt;

    AIO_ATTR_S stAioAttr;
    stAioAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
    stAioAttr.Bitwidth = AUDIO_BIT_WIDTH_16;
    stAioAttr.HwBitwidth = ES_AI_HW_BIT_WIDTH;
    stAioAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
    stAioAttr.FrmNum = 8;
    stAioAttr.PtNumPerFrm = SAMPLE_AUDIO_PTNUMPERFRM;
    stAioAttr.ChnCnt = 2;

    gs_bAioReSample = ES_TRUE;
    /* config ao resample attr if needed */
    if (ES_TRUE == gs_bAioReSample) {
        /* ai 48k -> 32k */
        enOutSampleRate = AUDIO_SAMPLE_RATE_32000;

        /* ao 32k -> 48k */
        enInSampleRate = AUDIO_SAMPLE_RATE_32000;
    } else {
        enInSampleRate = AUDIO_SAMPLE_RATE_BUTT;
        enOutSampleRate = AUDIO_SAMPLE_RATE_BUTT;
    }

    /* enable AI channle */
    s32Ret = SAMPLE_COMM_AUDIO_StartAi(AiCard, AiDev, &stAioAttr, enOutSampleRate, gs_bAioReSample, NULL, 0);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto AIAO_ERR3;
    }

    /* enable AO channle */
    s32AoChnCnt = stAioAttr.ChnCnt;
    AIO_ATTR_S aoAttr = stAioAttr;
    aoAttr.HwBitwidth = ES_AO_HW_BIT_WIDTH;
    s32Ret = SAMPLE_COMM_AUDIO_StartAo(AoCard, AoDev, s32AoChnCnt, &aoAttr, enInSampleRate, gs_bAioReSample);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto AIAO_ERR2;
    }

    /* bind AI to AO channle */
    if (ES_TRUE == gs_bUserGetMode) {
        s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAiAo(AiCard, AiDev, AoCard, AoDev);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AIAO_ERR1;
        }
    } else {
        s32Ret = SAMPLE_COMM_AUDIO_AoBindAi(AiCard, AiDev, AoCard, AoDev);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AIAO_ERR1;
        }
    }
    printf("ai(%d,%d) bind to ao(%d,%d) ok\n", AiCard, AiDev, AoCard, AoDev);

    if (ES_TRUE == gs_bAoVolumeCtrl) {
        s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAoVolCtrl(AoCard, AoDev);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto AIAO_ERR0;
        }
    }

    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

    if (ES_TRUE == gs_bAoVolumeCtrl) {
        s32Ret = SAMPLE_COMM_AUDIO_DestroyTrdAoVolCtrl(AoCard, AoDev);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            return ES_FAILURE;
        }
    }

AIAO_ERR0:
    if (ES_TRUE == gs_bUserGetMode) {
        s32Ret = SAMPLE_COMM_AUDIO_DestroyTrdAi(AiCard, AiDev);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
        }
    } else {
        s32Ret = SAMPLE_COMM_AUDIO_AoUnbindAi(AiCard, AiDev, AoCard, AoDev);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
        }
    }
AIAO_ERR1:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAo(AoCard, AoDev, s32AoChnCnt >> stAioAttr.Soundmode, gs_bAioReSample);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
    }
AIAO_ERR2:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAi(AiCard, AiDev, gs_bAioReSample, ES_FALSE);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
    }
AIAO_ERR3:
    return s32Ret;
}

/******************************************************************************
 * function : Ai -> Ao
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_AiVqeProcessAo(AUDIO_CARD AiCard, AUDIO_DEV AiDev, AUDIO_CARD AoCard, AUDIO_DEV AoDev)
{
    ES_S32 s32Ret;
    ES_S32 s32AoChnCnt;
    AIO_ATTR_S stAioAttr;
    AI_VQE_CONFIG_S stAiVqeAttr;
    AO_VQE_CONFIG_S stAoVqeAttr;
    ES_VOID* pAiVqeAttr = NULL;
    ES_S8 eqConfig[VQE_EQ_BAND_NUM] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    stAioAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
    stAioAttr.Bitwidth = AUDIO_BIT_WIDTH_16;
    stAioAttr.HwBitwidth = ES_AI_HW_BIT_WIDTH;
    stAioAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
    stAioAttr.FrmNum = 8;
    stAioAttr.PtNumPerFrm = SAMPLE_AUDIO_PTNUMPERFRM;
    stAioAttr.ChnCnt = 2;

    gs_bAioReSample = ES_FALSE;
    enInSampleRate = AUDIO_SAMPLE_RATE_BUTT;
    enOutSampleRate = AUDIO_SAMPLE_RATE_BUTT;

    if (1 == u32AiVqeType) {
        stAioAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
        stAioAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
        stAioAttr.ChnCnt = 2;
        memset(&stAiVqeAttr, 0, sizeof(AI_VQE_CONFIG_S));
        stAiVqeAttr.WorkSampleRate = AUDIO_SAMPLE_RATE_16000;
        stAiVqeAttr.FrameSamples = SAMPLE_AUDIO_PTNUMPERFRM;
        stAiVqeAttr.WorkState = VQE_WORKSTATE_COMMON;

        stAiVqeAttr.HpfConfig.UsrMode = ES_TRUE;
        stAiVqeAttr.HpfConfig.HpfFreq = 80;
        stAiVqeAttr.DrcConfig.UsrMode = ES_TRUE;
        stAiVqeAttr.DrcConfig.Threshold = -10;
        stAiVqeAttr.DrcConfig.Knee = 10;
        stAiVqeAttr.DrcConfig.Ratio = 2;
        stAiVqeAttr.DrcConfig.Attack = 10;
        stAiVqeAttr.DrcConfig.Release = 10;
        stAiVqeAttr.DrcConfig.PostGain = 0;
        memcpy(stAiVqeAttr.EqConfig.GaindB, eqConfig, VQE_EQ_BAND_NUM);
        stAiVqeAttr.AnrConfig.UsrMode = ES_FALSE;
        stAiVqeAttr.AgcConfig.UsrMode = ES_FALSE;

        stAiVqeAttr.OpenMask = AI_VQE_MASK_DRC | AI_VQE_MASK_EQ | AI_VQE_MASK_AGC | AI_VQE_MASK_HPF | AI_VQE_MASK_ANR;

        pAiVqeAttr = (ES_VOID*)&stAiVqeAttr;
    } else {
        pAiVqeAttr = ES_NULL;
    }

    stAoVqeAttr.WorkSampleRate = AUDIO_SAMPLE_RATE_16000;
    stAoVqeAttr.FrameSamples = SAMPLE_AUDIO_PTNUMPERFRM;
    stAoVqeAttr.WorkState = VQE_WORKSTATE_COMMON;

    stAoVqeAttr.HpfConfig.UsrMode = ES_FALSE;
    memcpy(stAoVqeAttr.EqConfig.GaindB, eqConfig, VQE_EQ_BAND_NUM);
    stAoVqeAttr.AnrConfig.UsrMode = ES_TRUE;
    stAoVqeAttr.AnrConfig.Mode = 1;
    stAoVqeAttr.AgcConfig.UsrMode = ES_TRUE;
    stAoVqeAttr.AgcConfig.TargetLevel = -10;
    stAoVqeAttr.AgcConfig.CompressionGain = 10;
    stAoVqeAttr.AgcConfig.LimiterEnable = 0;
    stAoVqeAttr.OpenMask = AO_VQE_MASK_AGC | AO_VQE_MASK_HPF | AO_VQE_MASK_ANR | AO_VQE_MASK_EQ;

    /********************************************
      step 1: start Ai
    ********************************************/
    s32Ret = SAMPLE_COMM_AUDIO_StartAi(AiCard, AiDev, &stAioAttr, enOutSampleRate, gs_bAioReSample, pAiVqeAttr,
                                       u32AiVqeType);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto VQE_ERR2;
    }

    /********************************************
      step 2: start Ao
    ********************************************/
    s32AoChnCnt = stAioAttr.ChnCnt;
    AIO_ATTR_S aoAttr = stAioAttr;
    aoAttr.HwBitwidth = ES_AO_HW_BIT_WIDTH;
    s32Ret = SAMPLE_COMM_AUDIO_StartAoWithVQE(AoCard, AoDev, s32AoChnCnt, &aoAttr, enInSampleRate, gs_bAioReSample,
                                              &stAoVqeAttr);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto VQE_ERR1;
    }

    /********************************************
      step 4: Ao bind Ai Chn
    ********************************************/
    s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAiAo(AiCard, AiDev, AoCard, AoDev);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        goto VQE_ERR0;
    }

    printf("bind ai(%d,%d) to ao(%d,%d) ok \n", AiCard, AiDev, AoCard, AoDev);
    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

    /********************************************
      step 6: exit the process
    ********************************************/
    s32Ret = SAMPLE_COMM_AUDIO_DestroyTrdAi(AiCard, AiDev);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
    }

VQE_ERR0:
    s32Ret = SAMPLE_COMM_AUDIO_StopAo(AoCard, AoDev, s32AoChnCnt >> stAioAttr.Soundmode, gs_bAioReSample);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
    }
VQE_ERR1:
    s32Ret |= SAMPLE_COMM_AUDIO_StopAi(AiCard, AiDev, gs_bAioReSample, ES_TRUE);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
    }
VQE_ERR2:
    return s32Ret;
}

/******************************************************************************
 * function : Ai -> Ao(Hdmi)
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_AiHdmiAo(AUDIO_CARD AiCard, AUDIO_DEV AiDev, AUDIO_CARD AoCard, AUDIO_DEV AoDev)
{
    ES_S32 s32Ret;
    AIO_ATTR_S stAioAttr;
    AIO_ATTR_S stHdmiAoAttr;

    stAioAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
    stAioAttr.Bitwidth = AUDIO_BIT_WIDTH_16;
    stAioAttr.HwBitwidth = ES_AI_HW_BIT_WIDTH;
    stAioAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
    stAioAttr.FrmNum = 30;
    stAioAttr.PtNumPerFrm = SAMPLE_AUDIO_PTNUMPERFRM;
    stAioAttr.ChnCnt = 2;

    stHdmiAoAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
    stHdmiAoAttr.Bitwidth = AUDIO_BIT_WIDTH_16;
    stHdmiAoAttr.HwBitwidth = ES_AO_HW_BIT_WIDTH;
    stHdmiAoAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
    stHdmiAoAttr.FrmNum = 30;
    stHdmiAoAttr.PtNumPerFrm = SAMPLE_AUDIO_PTNUMPERFRM;
    stHdmiAoAttr.ChnCnt = 2;

    gs_bAioReSample = ES_FALSE;

    /* enable AI channle */
    s32Ret = SAMPLE_COMM_AUDIO_StartAi(AiCard, AiDev, &stAioAttr, AUDIO_SAMPLE_RATE_BUTT, ES_FALSE, NULL, 0);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        return ES_FAILURE;
    }

    /* enable AO channle */
    s32Ret =
        SAMPLE_COMM_AUDIO_StartAo(AoCard, AoDev, stHdmiAoAttr.ChnCnt, &stHdmiAoAttr, stHdmiAoAttr.Samplerate, ES_FALSE);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        return ES_FAILURE;
    }

    /* AI to AO channel */
    s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAiAo(AiCard, AiDev, AoCard, AoDev);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        return ES_FAILURE;
    }

    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

    s32Ret = SAMPLE_COMM_AUDIO_DestroyTrdAi(AiCard, AiDev);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        return ES_FAILURE;
    }

    s32Ret = SAMPLE_COMM_AUDIO_StopAi(AiCard, AiDev, ES_FALSE, ES_FALSE);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        return ES_FAILURE;
    }

    s32Ret = SAMPLE_COMM_AUDIO_StopAo(AoCard, AoDev, stHdmiAoAttr.ChnCnt >> stHdmiAoAttr.Soundmode, ES_FALSE);
    if (s32Ret != ES_SUCCESS) {
        SAMPLE_DBG(s32Ret);
        return ES_FAILURE;
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : AI dual-DIE sample
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_DUAL_DIE_AI()
{
    AUDIO_CARD AiCard;
    AUDIO_DEV AiDev;
    ES_S32 s32Ret;
    FILE* pfd[2] = {NULL};
    ES_BOOL startFlag[2] = {ES_FALSE};
    AIO_ATTR_S stAioAttr;
    ES_S32 dieId = 0;
    ES_S32 AiDevId[2][2] = {{0, 0}, {3, 0}};

    ES_S32 numaAvail = ES_NUMA_Avaiable();
    if (!numaAvail) {
        printf("Only DIE 0 is supported on this hardware. Please run this case on dual-DIE hardware.\n");
        return ES_FAILURE;
    }

    stAioAttr.Samplerate = AUDIO_SAMPLE_RATE_48000;
    stAioAttr.Bitwidth = ES_AI_HW_BIT_WIDTH;
    stAioAttr.HwBitwidth = ES_AI_HW_BIT_WIDTH;
    stAioAttr.Soundmode = AUDIO_SOUND_MODE_STEREO;
    stAioAttr.ChnCnt = 2;
    stAioAttr.PtNumPerFrm = SAMPLE_AUDIO_PTNUMPERFRM;
    stAioAttr.FrmNum = 30;

    for (dieId = 0; dieId < 2; dieId++) {
        AiCard = AiDevId[dieId][0];
        AiDev = AiDevId[dieId][1];
        pfd[dieId] = SAMPLE_AUDIO_OpenAiFile(AiCard, AiDev);
        if (!pfd[dieId]) {
            SAMPLE_DBG(ES_FAILURE);
            goto EXIT;
        }

        s32Ret = SAMPLE_COMM_AUDIO_StartAi(AiCard, AiDev, &stAioAttr, AUDIO_SAMPLE_RATE_BUTT, ES_FALSE, NULL, 0);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto EXIT;
        }
        startFlag[dieId] = ES_TRUE;

        s32Ret = SAMPLE_COMM_AUDIO_CreatTrdAi(AiCard, AiDev, pfd[dieId]);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto EXIT;
        }

        printf("AI(%d, %d) start success\n", AiCard, AiDev);
    }

    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

EXIT:
    for (dieId = 0; dieId < 2; dieId++) {
        AiCard = AiDevId[dieId][0];
        AiDev = AiDevId[dieId][1];
        SAMPLE_COMM_AUDIO_DestroyTrdAi(AiCard, AiDev);

        if (startFlag[dieId]) {
            SAMPLE_COMM_AUDIO_StopAi(AiCard, AiDev, ES_FALSE, ES_FALSE);
        }

        if (pfd[dieId] != NULL) {
            fclose(pfd[dieId]);
            pfd[dieId] = NULL;
        }
    }
    return s32Ret;
}

/******************************************************************************
 * function : AENC dual-DIE sample
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_DUAL_DIE_AENC(ES_VOID)
{
    ES_S32 s32Ret = ES_FAILURE;
    AIO_ATTR_S stAioAttr;
    struct chunk_fmt stChunkFmt;
    FILE* pInFd[2] = {NULL};
    FILE* pOutFd[2] = {NULL};
    ES_BOOL startFlag[2] = {ES_FALSE};
    ES_S32 dieId = 0;

    ES_S32 numaAvail = ES_NUMA_Avaiable();
    if (!numaAvail) {
        printf("Only DIE 0 is supported on this hardware. Please run this case on dual-DIE hardware.\n");
        return ES_FAILURE;
    }

    for (dieId = 0; dieId < 2; dieId++) {
        pInFd[dieId] = SAMPLE_AUDIO_OpenAencWavFile();
        if (NULL == pInFd[dieId]) {
            SAMPLE_DBG(ES_FAILURE);
            goto EXIT;
        }

        s32Ret = parseWav(pInFd[dieId], &stChunkFmt);
        if (ES_SUCCESS != s32Ret) {
            SAMPLE_DBG(ES_FAILURE);
            goto EXIT;
        }

        stAioAttr.Bitwidth = convertBitsToBitWidth(stChunkFmt.bits_per_sample);
        stAioAttr.Soundmode = convertChnsToSoundMode(stChunkFmt.num_channels);
        stAioAttr.ChnCnt = stChunkFmt.num_channels;
        stAioAttr.Samplerate = (AUDIO_SAMPLE_RATE_E)stChunkFmt.sample_rate;
        stAioAttr.FrmNum = 30;
#ifndef ____ADPCM_DEC_SUPPORT__
        stAioAttr.PtNumPerFrm = AACLC_SAMPLES_PER_FRAME;
#else
        stAioAttr.PtNumPerFrm = ADPCM_SAMPLES_PER_FRAME;
#endif

        s32Ret = SAMPLE_COMM_AUDIO_StartAenc(dieId, &stAioAttr, gs_enPayloadType);
        if (ES_SUCCESS != s32Ret) {
            SAMPLE_DBG(s32Ret);
            goto EXIT;
        }
        startFlag[dieId] = ES_TRUE;

        pOutFd[dieId] = SAMPLE_AUDIO_OpenAencFile(dieId, gs_enPayloadType);
        if (NULL == pOutFd) {
            SAMPLE_DBG(s32Ret);
            goto EXIT;
        }

        /* Here we use dieid as the Aenc channel ID */
        s32Ret = SAMPLE_COMM_AUDIO_CreateTrdAenc(dieId, &stAioAttr, pInFd[dieId], pOutFd[dieId]);
        if (ES_SUCCESS != s32Ret) {
            SAMPLE_DBG(s32Ret);
            goto EXIT;
        }

        printf("aenc(%d) start ok\n", dieId);
    }

    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

EXIT:
    for (dieId = 0; dieId < 2; dieId++) {
        s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAenc(dieId);

        if (pOutFd[dieId] != NULL) {
            fclose(pOutFd[dieId]);
            pOutFd[dieId] = NULL;
        }

        if (startFlag[dieId]) {
            s32Ret |= SAMPLE_COMM_AUDIO_StopAenc(dieId);
        }

        if (pInFd[dieId] != NULL) {
            fclose(pInFd[dieId]);
            pInFd[dieId] = NULL;
        }
    }
    return s32Ret;
}

/******************************************************************************
 * function : ADEC dual-DIE sample
 ******************************************************************************/
ES_S32 SAMPLE_AUDIO_DUAL_DIE_ADEC(ES_VOID)
{
    ES_S32 s32Ret = ES_FAILURE;
    ADEC_CHN AdecChn = 0;
    FILE* pInFd[2] = {NULL};
    FILE* pOutFd[2] = {NULL};
    ES_BOOL startFlag[2] = {ES_FALSE};
    ES_S32 dieId = 0;

    ES_S32 numaAvail = ES_NUMA_Avaiable();
    if (!numaAvail) {
        printf("Only DIE 0 is supported on this hardware. Please run this case on dual-DIE hardware.\n");
        return ES_FAILURE;
    }

    for (dieId = 0; dieId < 2; dieId++) {
        pInFd[dieId] = SAMPLE_AUDIO_OpenAdecFile(dieId, gs_enPayloadType);
        if (!pInFd[dieId]) {
            SAMPLE_DBG(ES_FAILURE);
            goto EXIT;
        }

        pOutFd[dieId] = SAMPLE_AUDIO_OpenAdecOutputFile(dieId, gs_enPayloadType);
        if (!pOutFd[dieId]) {
            SAMPLE_DBG(ES_FAILURE);
            goto EXIT;
        }

        s32Ret = SAMPLE_COMM_AUDIO_StartAdec(dieId, gs_enPayloadType);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto EXIT;
        }
        startFlag[dieId] = ES_TRUE;

        s32Ret = SAMPLE_COMM_AUDIO_CreateTrdAdecGetData(dieId, pInFd[dieId], pOutFd[dieId], INVALID_AUDIO_CARD,
                                                        INVALID_AUDIO_DEV);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto EXIT;
        }

        /* Here we use dieid as the Adec channel ID */
        s32Ret = SAMPLE_COMM_AUDIO_CreateTrdAdec(dieId, pInFd[dieId], pOutFd[dieId]);
        if (s32Ret != ES_SUCCESS) {
            SAMPLE_DBG(s32Ret);
            goto EXIT;
        }

        printf("adec(%d) start ok\n", AdecChn);
    }

    printf("\nplease press twice ENTER to exit this sample\n");
    getchar();
    getchar();

EXIT:
    for (dieId = 0; dieId < 2; dieId++) {
        s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAdec(dieId);

        s32Ret |= SAMPLE_COMM_AUDIO_DestroyTrdAdecGetData(dieId);

        if (startFlag[dieId]) {
            s32Ret |= SAMPLE_COMM_AUDIO_StopAdec(dieId);
        }

        if (pOutFd[dieId] != NULL) {
            fclose(pOutFd[dieId]);
            pOutFd[dieId] = NULL;
        }

        if (pInFd[dieId] != NULL) {
            fclose(pInFd[dieId]);
            pInFd[dieId] = NULL;
        }
    }
    return s32Ret;
}

ES_VOID SAMPLE_AUDIO_Usage(ES_VOID)
{
    printf("Usage: ./sample_audio -m <test_mode> -i <input_card,input_device> -o <output_card,output_device>\n");
    printf("    Notice: -i & -o are optional, default using 0,0 for AI & AO, ");
    printf("1,0 or 2,0 for AO-HDMI(for dvb, evb board)\n");
    printf("    Notice: execute sample_audio as root user\n");
    printf("    Notice: ensure audio node is not occupied, the command for handling pulseaudio as:\n");
    printf("            systemctl --user stop pulseaudio.socket; systemctl --user stop pulseaudio; ");
    printf("            systemctl --user start pulseaudio\n");
    printf("    Notice: execute es_media_srv for using sys bind, it's necessary for execute sample_audio\n");
    printf("    Notice: execute modetest -M es_drm -D 0 -s 79@52:#0 to enable HDMI\n");
    printf("    test_mode option:\n");
    printf("    0: AI to AO, optional params: -i & -o\n");
    printf("    1: AI to AENC, then ADEC to AO, optional params: -i & -o\n");
    printf("    2: ADEC to AO, optional params: -o\n");
    printf("    3: AI-VQE to AO-VQE, optional params: -i & -o\n");
    printf("    4: AI to AO-HDMI, optional params: -i & -o\n");
    printf("    5: AI, optional params: -i\n");
    printf("    6: AO, optional params: -o\n");
    printf("    7: AENC\n");
    printf("    8: ADEC\n");
    printf("    9: AO dual-DIE sample\n");
    printf("    10: AI dual-DIE sample\n");
    printf("    11: AENC dual-DIE sample\n");
    printf("    12: ADEC dual-DIE sample\n");
}

/******************************************************************************
 * function : to process abnormal case
 ******************************************************************************/
void SAMPLE_AUDIO_HandleSig(ES_S32 signo)
{
    signal(SIGINT, SIG_IGN);
    signal(SIGTERM, SIG_IGN);

    if (SIGINT == signo || SIGTERM == signo) {
        SAMPLE_COMM_AUDIO_DestroyAllTrd();
        printf("\033[0;31mprogram exit abnormally!\033[0;39m\n");
    }

    exit(0);
}

ES_S32 SAMPLE_IS_SERVICE_RUNNING(const char* serviceName)
{
    char command[256];
    snprintf(command, sizeof(command), "pgrep %s > /dev/null 2>&1", serviceName);
    return (system(command) == 0);
}

/******************************************************************************
 * function : main
 ******************************************************************************/
ES_S32 main(int argc, char* argv[])
{
    ES_S32 s32Ret, opt;
    ES_S32 test_mode = -1;
    ES_U64 version;
    ES_CHAR versionStr[VERSION_NAME_MAXLEN];
    ES_CHAR* inputStr = NULL;
    ES_CHAR* outputStr = NULL;
    ES_CHAR cmd[128] = "ls /dev/snd | grep -q pcmC2D0p";
    AUDIO_CARD AiCard = SAMPLE_AUDIO_AI_CARD;
    AUDIO_DEV AiDev = SAMPLE_AUDIO_AI_DEV;
    AUDIO_CARD AoCard = SAMPLE_AUDIO_AO_CARD;
    AUDIO_DEV AoDev = SAMPLE_AUDIO_AO_DEV;
    const char* serviceName = "es_media_srv";

    while ((opt = getopt(argc, argv, "m:i:o:")) != -1) {
        switch (opt) {
        case 'm':
            test_mode = atoi(optarg);
            break;
        case 'i':
            inputStr = optarg;
            break;
        case 'o':
            outputStr = optarg;
            break;
        default:
            SAMPLE_AUDIO_Usage();
            return ES_FAILURE;
        }
    }
    if ((test_mode == -1) || (test_mode > 12)) {
        SAMPLE_AUDIO_Usage();
        return ES_FAILURE;
    }

    if (ES_Audio_GetVersion(&version) != ES_SUCCESS) {
        printf("get version failed\n");
        return ES_FAILURE;
    }
    if (ES_Audio_GetVersionString(versionStr, VERSION_NAME_MAXLEN) != ES_SUCCESS) {
        printf("get version string failed\n");
        return ES_FAILURE;
    }
    printf("ESSDK Audio version:%s\n", versionStr);

    if (!SAMPLE_IS_SERVICE_RUNNING(serviceName)) {
        printf("%s is not running. Please start it first.\n", serviceName);
        return 1;
    }

    signal(SIGINT, SAMPLE_AUDIO_HandleSig);
    signal(SIGTERM, SAMPLE_AUDIO_HandleSig);

#ifdef __ADPCM_DEC_SUPPORT__
    ES_S32 adpcmEncHandle = 0;
    ES_S32 adpcmDecHandle = 0;
    ES_AENC_AdpcmInit(&adpcmEncHandle);
    ES_ADEC_AdpcmInit(&adpcmDecHandle);
#endif

    switch (test_mode) {
    case 0:
        if ((get_device(inputStr, &AiCard, &AiDev) == ES_SUCCESS) &&
            (get_device(outputStr, &AoCard, &AoDev) == ES_SUCCESS)) {
            s32Ret = SAMPLE_AUDIO_AiAo(AiCard, AiDev, AoCard, AoDev);
        }
        break;
    case 1:
        if ((get_device(inputStr, &AiCard, &AiDev) == ES_SUCCESS) &&
            (get_device(outputStr, &AoCard, &AoDev) == ES_SUCCESS)) {
            s32Ret = SAMPLE_AUDIO_AiAenc(AiCard, AiDev, AoCard, AoDev);
        }
        break;
    case 2:
        if (get_device(outputStr, &AoCard, &AoDev) == ES_SUCCESS) {
            s32Ret = SAMPLE_AUDIO_AdecAo(AoCard, AoDev);
        }
        break;
    case 3:
        if ((get_device(inputStr, &AiCard, &AiDev) == ES_SUCCESS) &&
            (get_device(outputStr, &AoCard, &AoDev) == ES_SUCCESS)) {
            s32Ret = SAMPLE_AUDIO_AiVqeProcessAo(AiCard, AiDev, AoCard, AoDev);
        }
        break;
    case 4:
        if (system(cmd) != 0) {
            AoCard = SAMPLE_AUDIO_HDMI_CARD_DVB;
        } else {
            AoCard = SAMPLE_AUDIO_HDMI_CARD_EVB;
        }
        AoDev = SAMPLE_AUDIO_HDMI_DEV;
        if ((get_device(inputStr, &AiCard, &AiDev) == ES_SUCCESS) &&
            (get_device(outputStr, &AoCard, &AoDev) == ES_SUCCESS)) {
            s32Ret = SAMPLE_AUDIO_AiHdmiAo(AiCard, AiDev, AoCard, AoDev);
        }
        break;
    case 5:
        if (get_device(inputStr, &AiCard, &AiDev) == ES_SUCCESS) {
            s32Ret = SAMPLE_AUDIO_AI(AiCard, AiDev);
        }
        break;
    case 6:
        if (get_device(outputStr, &AoCard, &AoDev) == ES_SUCCESS) {
            s32Ret = SAMPLE_AUDIO_AO(AoCard, AoDev);
        }
        break;
    case 7:
        s32Ret = SAMPLE_AUDIO_AENC();
        break;
    case 8:
        s32Ret = SAMPLE_AUDIO_ADEC();
        break;
    case 9:
        s32Ret = SAMPLE_AUDIO_DUAL_DIE_AO();
        break;
    case 10:
        s32Ret = SAMPLE_AUDIO_DUAL_DIE_AI();
        break;
    case 11:
        s32Ret = SAMPLE_AUDIO_DUAL_DIE_AENC();
        break;
    case 12:
        s32Ret = SAMPLE_AUDIO_DUAL_DIE_ADEC();
        break;
    default:
        break;
    }

#ifdef __ADPCM_DEC_SUPPORT__
    ES_AENC_AdpcmDeInit(adpcmEncHandle);
    ES_ADEC_AdpcmDeInit(adpcmDecHandle);
#endif

    return s32Ret;
}
