// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sched.h>
#include <pthread.h>
#include <errno.h>
#include <signal.h>
#include <sys/select.h>
#include "sample_comm.h"
#include "es_audio.h"
#include "utils.h"
#include "es_sys.h"
#include "es-numa.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* End of #ifdef __cplusplus */

typedef struct tagSAMPLE_AENC_S {
    ES_BOOL bStart;
    pthread_t stAencPid;
    ES_S32 AeChn;
    ES_S32 AdChn;
    FILE* pOutFd;
    FILE* pInFd;
    AIO_ATTR_S stAioAttr;
    ES_BOOL bSendAdChn;
} SAMPLE_AENC_S;

typedef struct tagSAMPLE_AI_S {
    ES_BOOL bStart;
    ES_BOOL bVolCtrlStart;
    ES_S32 AiCard;
    ES_S32 AiDev;
    FILE* pfd;
    ES_S32 AencChn;
    ES_U32 AoCardId;
    ES_S32 AoDev;
    ES_BOOL bSendAenc;
    ES_BOOL bSendAo;
    pthread_t stAiPid;
    pthread_t stAiVolPid;
} SAMPLE_AI_S;

typedef struct tagSAMPLE_ADEC_S {
    ES_BOOL bStart;
    ES_BOOL bGetDataStart;
    ES_S32 AdChn;
    FILE* pInFd;
    FILE* pOutFd;
    pthread_t stAdPid;
    pthread_t stAdGetDataPid;
    AUDIO_CARD AoCardId;
    AUDIO_DEV AoDev;
} SAMPLE_ADEC_S;

typedef struct tagSAMPLE_AO_S {
    AUDIO_CARD AoCardId;
    AUDIO_DEV AoDev;
    FILE* pfd;
    ES_BOOL bStart;
    ES_BOOL bVolCtrlStart;
    ES_BOOL bPauseCtrlStart;
    pthread_t stAoPid;
    pthread_t stAoVolPid;
    pthread_t stAoPausePid;
    AIO_ATTR_S stAioAttr;
} SAMPLE_AO_S;

#define AI_MAX_CARD_NUM 16
#define AI_MAX_DEVICE_NUM 16
#define AO_MAX_CHN_NUM 1
#define AO_MAX_CARD_NUM 16
#define AO_MAX_DEVICE_NUM 16

static SAMPLE_AI_S gs_stSampleAi[AI_MAX_CARD_NUM * AI_MAX_DEVICE_NUM];
static SAMPLE_AENC_S gs_stSampleAenc[AENC_MAX_CHN_NUM];
static SAMPLE_ADEC_S gs_stSampleAdec[ADEC_MAX_CHN_NUM];
static SAMPLE_AO_S gs_stSampleAo[AO_MAX_CARD_NUM * AO_MAX_DEVICE_NUM];

static const ES_U8 bitWidthToBytes[AUDIO_BIT_WIDTH_BUTT] = {
    [AUDIO_BIT_WIDTH_8] = 1, [AUDIO_BIT_WIDTH_16] = 2, [AUDIO_BIT_WIDTH_24] = 3, [AUDIO_BIT_WIDTH_32] = 4};
static const ES_U8 soundModeToChannels[AUDIO_SOUND_MODE_BUTT] = {
    [AUDIO_SOUND_MODE_MONO] = 1, [AUDIO_SOUND_MODE_STEREO] = 2};

void setPthreadAffinityByCardId(int carID);

/******************************************************************************
 * function : get frame from Ai, send it  to Aenc or Ao
 ******************************************************************************/
void* SAMPLE_COMM_AUDIO_AiProc(void* parg)
{
    ES_S32 s32Ret;
    ES_S32 AiFd;
    SAMPLE_AI_S* pstAiCtl = (SAMPLE_AI_S*)parg;
    AUDIO_FRAME_S stFrame;
    AUDIO_AEC_FRAME_S stAecFrm;
    fd_set read_fds;
    struct timeval TimeoutVal;
    AI_CHN_PARAM_S stAiChnPara;
    ES_U8* data = NULL;

    setPthreadAffinityByCardId(pstAiCtl->AiCard);

    s32Ret = ES_AI_GetChnParam(pstAiCtl->AiCard, pstAiCtl->AiDev, &stAiChnPara);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: Get ai chn param failed\n", __FUNCTION__);
        goto AIPROC_EXIT;
    }

    stAiChnPara.UsrFrmDepth = 30;
    s32Ret = ES_AI_SetChnParam(pstAiCtl->AiCard, pstAiCtl->AiDev, &stAiChnPara);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: set ai chn param failed\n", __FUNCTION__);
        goto AIPROC_EXIT;
    }

    FD_ZERO(&read_fds);
    AiFd = ES_AI_GetFd(pstAiCtl->AiCard, pstAiCtl->AiDev);
    FD_SET(AiFd, &read_fds);

    while (pstAiCtl->bStart) {
        TimeoutVal.tv_sec = 1;
        TimeoutVal.tv_usec = 0;

        FD_ZERO(&read_fds);
        FD_SET(AiFd, &read_fds);
        s32Ret = select(AiFd + 1, &read_fds, NULL, NULL, &TimeoutVal);
        if (s32Ret < 0) {
            break;
        } else if (0 == s32Ret) {
            printf("%s: get ai frame select time out\n", __FUNCTION__);
            break;
        }

        if (FD_ISSET(AiFd, &read_fds)) {
            /* get frame from ai chn */
            memset(&stAecFrm, 0, sizeof(AUDIO_AEC_FRAME_S));
            s32Ret = ES_AI_GetFrame(pstAiCtl->AiCard, pstAiCtl->AiDev, &stFrame, NULL, ES_FALSE);
            if (ES_SUCCESS != s32Ret) {
#if 0
                printf("%s: ES_AI_GetFrame(%d, %d), failed with %#x!\n", \
                       __FUNCTION__, pstAiCtl->AiDev, pstAiCtl->AiChn, s32Ret);
                pstAiCtl->bStart = ES_FALSE;
                return NULL;
#else
                continue;
#endif
            }

            /* send frame to encoder */
            if (ES_TRUE == pstAiCtl->bSendAenc) {
                s32Ret = ES_AENC_SendFrame(pstAiCtl->AencChn, &stFrame, NULL);
                if (ES_SUCCESS != s32Ret) {
                    printf("%s: ES_AENC_SendFrame(%d), failed with %#x!\n", __FUNCTION__, pstAiCtl->AencChn, s32Ret);
                    goto AIPROC_EXIT;
                }
            }

            /* send frame to ao */
            if (ES_TRUE == pstAiCtl->bSendAo) {
                s32Ret = ES_AO_SendFrame(pstAiCtl->AoCardId, pstAiCtl->AoDev, &stFrame, 1000);
                if (ES_SUCCESS != s32Ret) {
                    printf("%s: ES_AO_SendFrame(%d, %d), failed with %#x!\n", __FUNCTION__, pstAiCtl->AoCardId,
                           pstAiCtl->AoDev, s32Ret);
                    goto AIPROC_EXIT;
                }
            }

            /* Save frame in file when not send to ao or aenc */
            if (ES_FALSE == pstAiCtl->bSendAenc && ES_FALSE == pstAiCtl->bSendAo) {
                if (!data) {
                    data = malloc(SAMPLE_AUDIO_MAX_PTNUMPERFRM * 4 * 2);  // max data len: 32bit 2channel
                    if (!data) {
                        printf("%s: alloc failed!\n", __FUNCTION__);
                        goto AIPROC_EXIT;
                    }
                }
                if (AUDIO_SOUND_MODE_MONO == stFrame.Soundmode) {
                    fwrite(stFrame.VirAddr, 1, stFrame.Len, pstAiCtl->pfd);
                } else if (AUDIO_SOUND_MODE_STEREO == stFrame.Soundmode) {
                    fwrite(stFrame.VirAddr, 1, stFrame.Len * 2, pstAiCtl->pfd);
                }
            }

            /* finally you must release the stream */
            s32Ret = ES_AI_ReleaseFrame(pstAiCtl->AiCard, pstAiCtl->AiDev, &stFrame, NULL);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_AI_ReleaseFrame(%d, %d), failed with %#x!\n", __FUNCTION__, pstAiCtl->AiCard,
                       pstAiCtl->AiDev, s32Ret);
                goto AIPROC_EXIT;
            }
        }
    }

AIPROC_EXIT:
    if (data) {
        free(data);
    }
    pstAiCtl->bStart = ES_FALSE;
    return NULL;
}

void setPthreadAffinityByCardId(int carID)
{
    ES_S32 numaAvail = ES_NUMA_Avaiable();
    if (!numaAvail) {
        printf("Only DIE 0 support on this hardware, run on DIE 0\n");
        return;
    }

    /* In a dual-DIE environment, cross-DIE data access may impact performance.
     * Therefore, we avoid cross-DIE access to improve overall operational efficiency.
     * Since cards 0–2 are on die 0 and cards 3–5 are on die 1, we bind threads to die 0 when opening cards 0–2,
     * and to die 1 when opening cards 3–5.
     */
    cpu_set_t cpuset;
    CPU_ZERO(&cpuset);
    if (carID < 3) {
        CPU_SET(0, &cpuset);
        CPU_SET(1, &cpuset);
        CPU_SET(2, &cpuset);
        CPU_SET(3, &cpuset);
    } else {
        CPU_SET(4, &cpuset);
        CPU_SET(5, &cpuset);
        CPU_SET(6, &cpuset);
        CPU_SET(7, &cpuset);
    }

    int ret = pthread_setaffinity_np(pthread_self(), sizeof(cpu_set_t), &cpuset);
    if (ret != 0) {
        printf("pthread_setaffinity_np failed, carID: %d, error: %s\n", carID, strerror(errno));
        return;
    }
}

void setPthreadAffinityByDieId(ES_S32 dieId)
{
    ES_S32 numaAvail = ES_NUMA_Avaiable();
    if (!numaAvail) {
        printf("Only DIE 0 support on this hardware, run on DIE 0\n");
        return;
    }

    /* In a dual-DIE environment, cross-DIE data access may impact performance.
     * Therefore, we avoid cross-DIE access to improve overall operational efficiency.
     */
    cpu_set_t cpuset;
    CPU_ZERO(&cpuset);
    if (dieId == 0) {
        CPU_SET(0, &cpuset);
        CPU_SET(1, &cpuset);
        CPU_SET(2, &cpuset);
        CPU_SET(3, &cpuset);
    } else {
        CPU_SET(4, &cpuset);
        CPU_SET(5, &cpuset);
        CPU_SET(6, &cpuset);
        CPU_SET(7, &cpuset);
    }

    int ret = pthread_setaffinity_np(pthread_self(), sizeof(cpu_set_t), &cpuset);
    if (ret != 0) {
        printf("pthread_setaffinity_np failed, dieId: %d, error: %s\n", dieId, strerror(errno));
        return;
    }
}

/******************************************************************************
 * function : get stream from file, send it Ao
 ******************************************************************************/
void* SAMPLE_COMM_AUDIO_AoProc(void* parg)
{
    ES_S32 s32Ret;
    SAMPLE_AO_S* pstAoCtl = (SAMPLE_AO_S*)parg;
    ES_U8* data = NULL;
    AIO_ATTR_S* attr = &pstAoCtl->stAioAttr;
    AUDIO_FRAME_S frame;
    int32_t framesize = 0;
    int32_t samples = attr->PtNumPerFrm;
    int32_t readnum;

    setPthreadAffinityByCardId(pstAoCtl->AoCardId);

    memset(&frame, 0, sizeof(frame));
    frame.Bitwidth = attr->Bitwidth;
    frame.Soundmode = attr->Soundmode;
    framesize = attr->ChnCnt * bitWidthToBytes[attr->Bitwidth];
    frame.Len = samples * framesize / attr->ChnCnt;
    if (attr->ChnCnt == 2) {
        frame.VirAddr = (ES_U8*)malloc(frame.Len * 2);
    } else {
        frame.VirAddr = (ES_U8*)malloc(frame.Len);
    }
    if (!frame.VirAddr) {
        printf("frame.VirAddr malloc failed\n");
        goto EXIT;
    }

    FILE* file = pstAoCtl->pfd;
    while (pstAoCtl->bStart) {
        readnum = fread(frame.VirAddr, 1, frame.Len * attr->ChnCnt, file);
        if (readnum <= 0) {
            break;
        }

        s32Ret = ES_AO_SendFrame(pstAoCtl->AoCardId, pstAoCtl->AoDev, &frame, -1);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AO_SendFrame(%d, %d), failed with %#x!\n", __FUNCTION__, pstAoCtl->AoCardId, pstAoCtl->AoDev,
                   s32Ret);
            break;
        }
    }

EXIT:
    if (frame.VirAddr) {
        free(frame.VirAddr);
    }
    if (data) {
        free(data);
    }
    pstAoCtl->bStart = ES_FALSE;
    printf("AO(%d, %d) play done\n", pstAoCtl->AoCardId, pstAoCtl->AoDev);
    return NULL;
}

/******************************************************************************
 * function : get stream from Aenc, send it  to Adec & save it to file
 ******************************************************************************/
void* SAMPLE_COMM_AUDIO_AencProc(void* parg)
{
    ES_S32 s32Ret;
    ES_S32 AencFd;
    SAMPLE_AENC_S* pstAencCtl = (SAMPLE_AENC_S*)parg;
    AUDIO_STREAM_S stStream;
    AUDIO_FRAME_S stFrame;
    fd_set read_fds;
    struct timeval TimeoutVal;
    ES_S32 dieId = 0;

    memset(&stFrame, 0x00, sizeof(stFrame));
    FD_ZERO(&read_fds);
    AencFd = ES_AENC_GetFd(pstAencCtl->AeChn);
    FD_SET(AencFd, &read_fds);

    /* Since we previously used dieid as the channel ID, here we obtain dieid from the channel ID. */
    dieId = pstAencCtl->AeChn;

    /* Here we place multiple encodings on different dies to provide parallel performance. */
    setPthreadAffinityByDieId(dieId);

    while (pstAencCtl->bStart) {
        if (pstAencCtl->pInFd) {
            if (NULL == stFrame.VirAddr) {
                stFrame.Bitwidth = pstAencCtl->stAioAttr.Bitwidth;
                stFrame.Soundmode = pstAencCtl->stAioAttr.Soundmode;
                stFrame.Len = pstAencCtl->stAioAttr.PtNumPerFrm * bitWidthToBytes[stFrame.Bitwidth];
                if (AUDIO_SOUND_MODE_STEREO == stFrame.Soundmode) {
                    stFrame.VirAddr = malloc(stFrame.Len * 2);
                } else {
                    stFrame.VirAddr = malloc(stFrame.Len);
                }
                if (NULL == stFrame.VirAddr) {
                    printf("%s: viraddr alloc failed\n", __FUNCTION__);
                    goto AENCPROC_EXIT;
                }
            }

            s32Ret = fread(stFrame.VirAddr, 1, stFrame.Len * soundModeToChannels[stFrame.Soundmode], pstAencCtl->pInFd);
            if (s32Ret <= 0) {
                printf("%s: file end\n", __FUNCTION__);
                goto AENCPROC_EXIT;
            }

            s32Ret = ES_AENC_SendFrame(pstAencCtl->AeChn, &stFrame, NULL);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: aenc send frame failed\n", __FUNCTION__);
                goto AENCPROC_EXIT;
            }
        }

        TimeoutVal.tv_sec = 1;
        TimeoutVal.tv_usec = 0;

        FD_ZERO(&read_fds);
        FD_SET(AencFd, &read_fds);
        s32Ret = select(AencFd + 1, &read_fds, NULL, NULL, &TimeoutVal);
        if (s32Ret < 0) {
            break;
        } else if (0 == s32Ret) {
            printf("%s: get aenc stream select time out\n", __FUNCTION__);
            break;
        }

        if (FD_ISSET(AencFd, &read_fds)) {
            /* get stream from aenc chn */
            s32Ret = ES_AENC_GetStream(pstAencCtl->AeChn, &stStream, ES_FALSE);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_AENC_GetStream(%d), failed with %#x!\n", __FUNCTION__, pstAencCtl->AeChn, s32Ret);
                goto AENCPROC_EXIT;
            }

            /* send stream to decoder and play for testing */
            if (ES_TRUE == pstAencCtl->bSendAdChn) {
                s32Ret = ES_ADEC_SendStream(pstAencCtl->AdChn, &stStream, ES_TRUE);
                if (ES_SUCCESS != s32Ret) {
                    printf("%s: ES_ADEC_SendStream(%d), failed with %#x!\n", __FUNCTION__, pstAencCtl->AdChn, s32Ret);
                    goto AENCPROC_EXIT;
                }
            }

            /* save audio stream to file */
            (ES_VOID) fwrite(stStream.Stream, 1, stStream.Len, pstAencCtl->pOutFd);

            fflush(pstAencCtl->pOutFd);

            /* finally you must release the stream */
            s32Ret = ES_AENC_ReleaseStream(pstAencCtl->AeChn, &stStream);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_AENC_ReleaseStream(%d), failed with %#x!\n", __FUNCTION__, pstAencCtl->AeChn, s32Ret);
                goto AENCPROC_EXIT;
            }
        }
    }

AENCPROC_EXIT:
    if (stFrame.VirAddr) {
        free(stFrame.VirAddr);
    }
    pstAencCtl->bStart = ES_FALSE;
    return NULL;
}

/******************************************************************************
 * function : get stream from file, and send it  to Adec
 ******************************************************************************/
void* SAMPLE_COMM_AUDIO_AdecProc(void* parg)
{
    ES_S32 s32Ret;
    AUDIO_STREAM_S stAudioStream;
    ES_U32 u32Len = 640;
    ES_U32 u32ReadLen;
    ES_S32 s32AdecChn;
    ES_U8* pu8AudioStream = NULL;
    SAMPLE_ADEC_S* pstAdecCtl = (SAMPLE_ADEC_S*)parg;
    FILE* pfd = pstAdecCtl->pInFd;
    s32AdecChn = pstAdecCtl->AdChn;

    /* Since we previously used dieid as the channel ID, here we obtain dieid from the channel ID,
     * Here we place multiple decodings on different dies to provide parallel performance.
     */
    setPthreadAffinityByDieId(s32AdecChn);

    pu8AudioStream = (ES_U8*)malloc(sizeof(ES_U8) * MAX_AUDIO_STREAM_LEN);
    if (NULL == pu8AudioStream) {
        printf("%s: malloc failed!\n", __FUNCTION__);
        return NULL;
    }

    while (ES_TRUE == pstAdecCtl->bStart) {
        /* read from file */
        stAudioStream.Stream = pu8AudioStream;
        u32ReadLen = fread(stAudioStream.Stream, 1, u32Len, pfd);
        if (u32ReadLen <= 0) {
            s32Ret = ES_ADEC_SendEndOfStream(s32AdecChn, ES_FALSE);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_ADEC_SendEndOfStream failed!\n", __FUNCTION__);
            }
            // (ES_VOID) fseek(pfd, 0, SEEK_SET); /*read file again*/
            break;
        }

        /* here only demo adec streaming sending mode, but pack sending mode is commended */
        stAudioStream.Len = u32ReadLen;
        s32Ret = ES_ADEC_SendStream(s32AdecChn, &stAudioStream, ES_TRUE);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_ADEC_SendStream(%d) failed with %#x!\n", __FUNCTION__, s32AdecChn, s32Ret);
            break;
        }
    }
    free(pu8AudioStream);
    pu8AudioStream = NULL;
    pstAdecCtl->bStart = ES_FALSE;

    return NULL;
}

/******************************************************************************
 * function : get stream from file, and send it to Adec
 ******************************************************************************/
void* SAMPLE_COMM_AUDIO_AdecGetDataProc(void* parg)
{
    ES_S32 s32Ret;
    SAMPLE_ADEC_S* pstAdecCtl = (SAMPLE_ADEC_S*)parg;
    FILE* pfd = pstAdecCtl->pOutFd;
    ES_S32 s32AdecChn = pstAdecCtl->AdChn;
    AUDIO_CARD AoCardId = pstAdecCtl->AoCardId;
    AUDIO_DEV AoDev = pstAdecCtl->AoDev;
    AUDIO_FRAME_INFO_S frame_info;
    AUDIO_FRAME_S frame;
    frame_info.Frame = &frame;

    while (ES_TRUE == pstAdecCtl->bGetDataStart) {
        s32Ret = ES_ADEC_GetFrame(s32AdecChn, &frame_info, ES_FALSE);
        if (s32Ret != ES_SUCCESS) {
            continue;
        }

        if (pfd) {
            fwrite(frame_info.Frame->VirAddr, 1, frame_info.Frame->Len * 2, pfd);
            fflush(pfd);
        }

        if ((AoDev >= 0)) {
            s32Ret = ES_AO_SendFrame(AoCardId, AoDev, &frame, -1);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_AO_SendFrame(%d, %d), failed with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
            }
        }

        s32Ret |= ES_ADEC_ReleaseFrame(s32AdecChn, &frame_info);
        if (s32Ret != ES_SUCCESS) {
            printf("SAMPLE_COMM_AUDIO_AdecGetDataProc failed s32Ret:%d\n", s32Ret);
            break;
        }
    }
    pstAdecCtl->bGetDataStart = ES_FALSE;

    return NULL;
}

/******************************************************************************
 * function : set ao volume
 ******************************************************************************/
void* SAMPLE_COMM_AUDIO_AoVolProc(void* parg)
{
    ES_S32 s32Ret;
    ES_S32 s32Volume;
    AUDIO_CARD AoCardId;
    AUDIO_DEV AoDev;
    AUDIO_FADE_S stFade;
    SAMPLE_AO_S* pstAoCtl = (SAMPLE_AO_S*)parg;
    AoDev = pstAoCtl->AoDev;
    AoCardId = pstAoCtl->AoCardId;

    while (pstAoCtl->bVolCtrlStart) {
        for (s32Volume = 0; s32Volume <= 6; s32Volume++) {
            s32Ret = ES_AO_SetVolume(AoCardId, AoDev, s32Volume);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_AO_SetVolume(%d,%d), failed with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
            }
            printf("\rset volume %d          ", s32Volume);
            fflush(stdout);
            sleep(2);
        }

        for (s32Volume = 5; s32Volume >= -15; s32Volume--) {
            s32Ret = ES_AO_SetVolume(AoCardId, AoDev, s32Volume);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_AO_SetVolume(%d,%d), failed with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
            }
            printf("\rset volume %d          ", s32Volume);
            fflush(stdout);
            sleep(2);
        }

        for (s32Volume = -14; s32Volume <= 0; s32Volume++) {
            s32Ret = ES_AO_SetVolume(AoCardId, AoDev, s32Volume);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_AO_SetVolume(%d,%d), failed with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
            }
            printf("\rset volume %d          ", s32Volume);
            fflush(stdout);
            sleep(2);
        }

        stFade.Fade = ES_TRUE;
        stFade.FadeInRate = 2000;
        stFade.FadeOutRate = 2000;

        s32Ret = ES_AO_SetMute(AoCardId, AoDev, ES_TRUE, &stFade);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AO_SetVolume(%d,%d), failed with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
        }
        printf("\rset Ao mute            ");
        fflush(stdout);
        sleep(2);

        s32Ret = ES_AO_SetMute(AoCardId, AoDev, ES_FALSE, &stFade);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AO_SetVolume(%d,%d), failed with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
        }
        printf("\rset Ao unmute          ");
        fflush(stdout);
        sleep(2);
    }

    return NULL;
}

/******************************************************************************
 * function : set ao volume
 ******************************************************************************/
void* SAMPLE_COMM_AUDIO_AoPauseProc(void* parg)
{
    ES_S32 s32Ret;
    AUDIO_CARD AoCardId;
    AUDIO_DEV AoDev;
    SAMPLE_AO_S* pstAoCtl = (SAMPLE_AO_S*)parg;
    AoDev = pstAoCtl->AoDev;
    AoCardId = pstAoCtl->AoCardId;

    /* ensure music is playing */
    sleep(3);
    /* ao is running */
    if (pstAoCtl->bStart) {
        s32Ret = ES_AO_PauseChn(pstAoCtl->AoCardId, pstAoCtl->AoDev);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AO_PauseChn(%d,%d), failed with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
        } else {
            printf("%s: ES_AO_PauseChn(%d,%d), success with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
        }
    }

    sleep(3);
    /* ao is playing */
    if (pstAoCtl->bStart) {
        s32Ret = ES_AO_ResumeChn(pstAoCtl->AoCardId, pstAoCtl->AoDev);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AO_ResumeChn(%d,%d), failed with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
        } else {
            printf("%s: ES_AO_ResumeChn(%d,%d), success with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
        }
    }

    pstAoCtl->bPauseCtrlStart = ES_FALSE;
    return NULL;
}

/******************************************************************************
 * function : set ai volume
 ******************************************************************************/
void* SAMPLE_COMM_AUDIO_AiVolProc(void* parg)
{
    ES_S32 s32Ret;
    ES_S32 s32Volume;
    AUDIO_CARD AiCard;
    AUDIO_DEV AiDev;
    SAMPLE_AI_S* pstAiCtl = (SAMPLE_AI_S*)parg;
    AiDev = pstAiCtl->AiDev;
    AiCard = pstAiCtl->AiCard;

    while (pstAiCtl->bVolCtrlStart) {
        for (s32Volume = 0; s32Volume <= 6; s32Volume++) {
            s32Ret = ES_AI_SetVolume(AiCard, AiDev, s32Volume);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_AI_SetVolume(%d), failed with %#x!\n", __FUNCTION__, AiDev, s32Ret);
            }
            printf("\rset volume %d          ", s32Volume);
            fflush(stdout);
            sleep(2);
        }

        for (s32Volume = 5; s32Volume >= -15; s32Volume--) {
            s32Ret = ES_AI_SetVolume(AiCard, AiDev, s32Volume);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_AI_SetVolume(%d), failed with %#x!\n", __FUNCTION__, AiDev, s32Ret);
            }
            printf("\rset volume %d          ", s32Volume);
            fflush(stdout);
            sleep(2);
        }

        for (s32Volume = -14; s32Volume <= 0; s32Volume++) {
            s32Ret = ES_AI_SetVolume(AiCard, AiDev, s32Volume);
            if (ES_SUCCESS != s32Ret) {
                printf("%s: ES_AI_SetVolume(%d), failed with %#x!\n", __FUNCTION__, AiDev, s32Ret);
            }
            printf("\rset volume %d          ", s32Volume);
            fflush(stdout);
            sleep(2);
        }
    }

    return NULL;
}

/******************************************************************************
 * function : other ai set
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_AiSetting(AUDIO_CARD AiCard, AUDIO_DEV AiDev, const AUDIO_SAVE_FILE_INFO_S* FileInfo,
                                   ES_BOOL bSaveFile, AUDIO_TRACK_MODE_E enTrackMode, ES_BOOL bTrackMode)
{
    ES_S32 s32Ret = ES_SUCCESS;

    if (bSaveFile) {
        s32Ret |= ES_AI_SaveFile(AiCard, AiDev, FileInfo);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AI_SaveFile(%d), failed with %#x!\n", __FUNCTION__, AiDev, s32Ret);
        }
    }

    if (bTrackMode) {
        s32Ret |= ES_AI_SetTrackMode(AiCard, AiDev, enTrackMode);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AI_SetTrackMode(%d), failed with %#x!\n", __FUNCTION__, AiDev, s32Ret);
        }
    }

    return s32Ret;
}

/******************************************************************************
 * function : Create the thread to get frame from ai and save file
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_CreatTrdAi(AUDIO_CARD AiCard, AUDIO_DEV AiDev, FILE* pAiFd)
{
    SAMPLE_AI_S* pstAi = &gs_stSampleAi[AiCard << 4 | AiDev];
    pstAi->AiDev = AiDev;
    pstAi->AiCard = AiCard;
    pstAi->bSendAenc = ES_FALSE;
    pstAi->bSendAo = ES_FALSE;
    pstAi->pfd = pAiFd;
    pstAi->bStart = ES_TRUE;
    pthread_create(&pstAi->stAiPid, 0, SAMPLE_COMM_AUDIO_AiProc, pstAi);

    return ES_SUCCESS;
}

/******************************************************************************
 * function : other ao set
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_AoSetting(AUDIO_CARD AoCardId, AUDIO_DEV AoDev, const AUDIO_SAVE_FILE_INFO_S* FileInfo,
                                   ES_BOOL bSaveFile, AUDIO_TRACK_MODE_E enTrackMode, ES_BOOL bTrackMode)
{
    ES_S32 s32Ret = ES_SUCCESS;

    if (bSaveFile) {
        s32Ret |= ES_AO_SaveFile(AoCardId, AoDev, FileInfo);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AI_SaveFile(%d,%d), failed with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
        }
    }

    if (bTrackMode) {
        s32Ret |= ES_AO_SetTrackMode(AoCardId, AoDev, enTrackMode);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AI_SetTrackMode(%d,%d), failed with %#x!\n", __FUNCTION__, AoCardId, AoDev, s32Ret);
        }
    }

    return s32Ret;
}

/******************************************************************************
 * function : Create the thread to get frame from file and send ao
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_CreatTrdAo(AUDIO_CARD AoCardId, AUDIO_DEV AoDev, AIO_ATTR_S* pstAioAttr, FILE* pAoFd)
{
    SAMPLE_AO_S* pstAo = NULL;

    pstAo = &gs_stSampleAo[AoCardId * AO_MAX_DEVICE_NUM + AoDev];
    pstAo->AoCardId = AoCardId;
    pstAo->AoDev = AoDev;
    pstAo->pfd = pAoFd;
    pstAo->bStart = ES_TRUE;
    pstAo->stAioAttr = *pstAioAttr;
    pthread_create(&pstAo->stAoPid, 0, SAMPLE_COMM_AUDIO_AoProc, pstAo);

    return ES_SUCCESS;
}

ES_S32 SAMPLE_COMM_AUDIO_CreateTrdAenc(AENC_CHN AencChn, AIO_ATTR_S* pstAioAttr, FILE* pInFd, FILE* pOutFd)
{
    SAMPLE_AENC_S* pstAenc = NULL;

    pstAenc = &gs_stSampleAenc[AencChn];
    pstAenc->AeChn = AencChn;
    pstAenc->bSendAdChn = ES_FALSE;
    pstAenc->pInFd = pInFd;
    pstAenc->pOutFd = pOutFd;
    pstAenc->bStart = ES_TRUE;
    if (pstAioAttr) {
        pstAenc->stAioAttr = *pstAioAttr;
    }
    pthread_create(&pstAenc->stAencPid, 0, SAMPLE_COMM_AUDIO_AencProc, pstAenc);

    return ES_SUCCESS;
}

ES_S32 SAMPLE_COMM_AUDIO_CreateTrdAdec(ADEC_CHN AdecChn, FILE* pInFd, FILE* pOutFd)
{
    SAMPLE_ADEC_S* pstAdec = NULL;

    pstAdec = &gs_stSampleAdec[AdecChn];
    pstAdec->AdChn = AdecChn;
    pstAdec->pInFd = pInFd;
    pstAdec->pOutFd = pOutFd;
    pstAdec->bStart = ES_TRUE;
    pthread_create(&pstAdec->stAdPid, 0, SAMPLE_COMM_AUDIO_AdecProc, pstAdec);

    return ES_SUCCESS;
}

ES_S32 SAMPLE_COMM_AUDIO_CreateTrdAdecGetData(ADEC_CHN AdecChn, FILE* pInFd, FILE* pOutFd, AUDIO_CARD AoCardId,
                                              AUDIO_DEV AoDev)
{
    SAMPLE_ADEC_S* pstAdec = NULL;

    pstAdec = &gs_stSampleAdec[AdecChn];
    pstAdec->AdChn = AdecChn;
    pstAdec->pInFd = pInFd;
    pstAdec->pOutFd = pOutFd;
    pstAdec->bGetDataStart = ES_TRUE;
    pstAdec->AoCardId = AoCardId;
    pstAdec->AoDev = AoDev;
    pthread_create(&pstAdec->stAdGetDataPid, 0, SAMPLE_COMM_AUDIO_AdecGetDataProc, pstAdec);

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Create the thread to get frame from ai and send to ao
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_CreatTrdAiAo(AUDIO_CARD AiCard, AUDIO_DEV AiDev, AUDIO_CARD AoCard, AUDIO_DEV AoDev)
{
    SAMPLE_AI_S* pstAi = NULL;

    pstAi = &gs_stSampleAi[AiCard << 4 | AiDev];
    pstAi->bSendAenc = ES_FALSE;
    pstAi->bSendAo = ES_TRUE;
    pstAi->bStart = ES_TRUE;
    pstAi->AiDev = AiDev;
    pstAi->AiCard = AiCard;
    pstAi->AoCardId = AoCard;
    pstAi->AoDev = AoDev;
    pthread_create(&pstAi->stAiPid, 0, SAMPLE_COMM_AUDIO_AiProc, pstAi);

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Create the thread to get frame from ai and send to aenc
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_CreatTrdAiAenc(AUDIO_CARD AiCard, AUDIO_DEV AiDev, AENC_CHN AeChn)
{
    SAMPLE_AI_S* pstAi = NULL;

    pstAi = &gs_stSampleAi[AiCard << 4 | AiDev];
    pstAi->bSendAenc = ES_TRUE;
    pstAi->bSendAo = ES_FALSE;
    pstAi->bStart = ES_TRUE;
    pstAi->AiDev = AiDev;
    pstAi->AiCard = AiCard;
    pstAi->AencChn = AeChn;
    pthread_create(&pstAi->stAiPid, 0, SAMPLE_COMM_AUDIO_AiProc, pstAi);

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Create the thread to get stream from aenc and send to adec
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_CreatTrdAencAdec(AENC_CHN AeChn, ADEC_CHN AdChn, FILE* pAecFd)
{
    SAMPLE_AENC_S* pstAenc = NULL;

    if (NULL == pAecFd) {
        return ES_FAILURE;
    }

    pstAenc = &gs_stSampleAenc[AeChn];
    pstAenc->AeChn = AeChn;
    pstAenc->AdChn = AdChn;
    pstAenc->bSendAdChn = ES_TRUE;
    pstAenc->pInFd = NULL;
    pstAenc->pOutFd = pAecFd;
    pstAenc->bStart = ES_TRUE;
    pthread_create(&pstAenc->stAencPid, 0, SAMPLE_COMM_AUDIO_AencProc, pstAenc);

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Create the thread to get stream from file and send to adec
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_CreatTrdFileAdec(ADEC_CHN AdChn, FILE* pAdcFd)
{
    SAMPLE_ADEC_S* pstAdec = NULL;

    if (NULL == pAdcFd) {
        return ES_FAILURE;
    }

    pstAdec = &gs_stSampleAdec[AdChn];
    pstAdec->AdChn = AdChn;
    pstAdec->pInFd = pAdcFd;
    pstAdec->bStart = ES_TRUE;
    pthread_create(&pstAdec->stAdPid, 0, SAMPLE_COMM_AUDIO_AdecProc, pstAdec);

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Create the thread to set Ai volume
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_CreatTrdAiVolCtrl(AUDIO_CARD AiCard, AUDIO_DEV AiDev)
{
    SAMPLE_AI_S* pstAiCtl = NULL;

    pstAiCtl = &gs_stSampleAi[AiCard << 4 | AiDev];
    if (ES_TRUE != pstAiCtl->bStart) {
        pstAiCtl->AiDev = AiDev;
        pstAiCtl->AiCard = AiCard;
    }
    pstAiCtl->bVolCtrlStart = ES_TRUE;
    pthread_create(&pstAiCtl->stAiVolPid, 0, SAMPLE_COMM_AUDIO_AiVolProc, pstAiCtl);

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Create the thread to set Ao volume
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_CreatTrdAoVolCtrl(AUDIO_CARD AoCardId, AUDIO_DEV AoDev)
{
    SAMPLE_AO_S* pstAoCtl = NULL;

    pstAoCtl = &gs_stSampleAo[AoCardId * AO_MAX_DEVICE_NUM + AoDev];
    if (ES_TRUE != pstAoCtl->bStart) {
        pstAoCtl->AoCardId = AoCardId;
        pstAoCtl->AoDev = AoDev;
    }
    pstAoCtl->bVolCtrlStart = ES_TRUE;
    pthread_create(&pstAoCtl->stAoVolPid, 0, SAMPLE_COMM_AUDIO_AoVolProc, pstAoCtl);

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Create the thread to set Ao volume
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_CreatTrdAoPauseCtrl(AUDIO_CARD AoCardId, AUDIO_DEV AoDev)
{
    SAMPLE_AO_S* pstAoCtl = NULL;

    pstAoCtl = &gs_stSampleAo[AoCardId * AO_MAX_DEVICE_NUM + AoDev];
    if (ES_TRUE != pstAoCtl->bStart) {
        pstAoCtl->AoCardId = AoCardId;
        pstAoCtl->AoDev = AoDev;
    }
    pstAoCtl->bPauseCtrlStart = ES_TRUE;
    pthread_create(&pstAoCtl->stAoPausePid, 0, SAMPLE_COMM_AUDIO_AoPauseProc, pstAoCtl);

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the thread ao
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyTrdAo(AUDIO_CARD AoCardId, AUDIO_DEV AoDev)
{
    SAMPLE_AO_S* pstAo = NULL;

    pstAo = &gs_stSampleAo[AoCardId * AO_MAX_DEVICE_NUM + AoDev];
    if (pstAo->bStart) {
        pstAo->bStart = ES_FALSE;
        pthread_join(pstAo->stAoPid, 0);
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the thread to get frame from ai and send to ao or aenc
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyTrdAi(AUDIO_CARD AiCard, AUDIO_DEV AiDev)
{
    SAMPLE_AI_S* pstAi = NULL;

    pstAi = &gs_stSampleAi[AiCard << 4 | AiDev];
    if (pstAi->bStart) {
        pstAi->bStart = ES_FALSE;
        pthread_join(pstAi->stAiPid, 0);
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the thread aenc
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyTrdAenc(AENC_CHN AencChn)
{
    SAMPLE_AENC_S* pstAenc = NULL;

    pstAenc = &gs_stSampleAenc[AencChn];
    if (pstAenc->bStart) {
        pstAenc->bStart = ES_FALSE;
        pthread_join(pstAenc->stAencPid, 0);
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the thread adec
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyTrdAdec(ADEC_CHN AdecChn)
{
    SAMPLE_ADEC_S* pstAdec = NULL;

    pstAdec = &gs_stSampleAdec[AdecChn];
    if (pstAdec->bStart) {
        pstAdec->bStart = ES_FALSE;
        pthread_join(pstAdec->stAdPid, 0);
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the get data thread adec
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyTrdAdecGetData(ADEC_CHN AdecChn)
{
    SAMPLE_ADEC_S* pstAdec = NULL;

    pstAdec = &gs_stSampleAdec[AdecChn];
    if (pstAdec->bGetDataStart) {
        pstAdec->bGetDataStart = ES_FALSE;
        pthread_join(pstAdec->stAdGetDataPid, 0);
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the thread to get stream from aenc and send to adec
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyTrdAencAdec(AENC_CHN AeChn)
{
    SAMPLE_AENC_S* pstAenc = NULL;

    pstAenc = &gs_stSampleAenc[AeChn];
    if (pstAenc->bStart) {
        pstAenc->bStart = ES_FALSE;
        pthread_join(pstAenc->stAencPid, 0);
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the thread to get stream from file and send to adec
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyTrdFileAdec(ADEC_CHN AdChn)
{
    SAMPLE_ADEC_S* pstAdec = NULL;

    pstAdec = &gs_stSampleAdec[AdChn];
    if (pstAdec->bStart) {
        pstAdec->bStart = ES_FALSE;
        pthread_join(pstAdec->stAdPid, 0);
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the thread to set Ai volume
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyTrdAiVolCtrl(AUDIO_CARD AiCard, AUDIO_DEV AiDev)
{
    SAMPLE_AI_S* pstAiCtl = NULL;

    pstAiCtl = &gs_stSampleAi[AiCard << 4 | AiDev];
    if (pstAiCtl->bVolCtrlStart) {
        pstAiCtl->bVolCtrlStart = ES_FALSE;
        pthread_cancel(pstAiCtl->stAiVolPid);
        pthread_join(pstAiCtl->stAiVolPid, 0);
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the thread to set Ao volume
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyTrdAoVolCtrl(AUDIO_CARD AoCardId, AUDIO_DEV AoDev)
{
    SAMPLE_AO_S* pstAoCtl = NULL;

    pstAoCtl = &gs_stSampleAo[AoCardId * AO_MAX_DEVICE_NUM + AoDev];
    if (pstAoCtl->bVolCtrlStart) {
        pstAoCtl->bVolCtrlStart = ES_FALSE;
        pthread_cancel(pstAoCtl->stAoVolPid);
        pthread_join(pstAoCtl->stAoVolPid, 0);
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the thread to set Ao volume
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyTrdAoPauseCtrl(AUDIO_CARD AoCardId, AUDIO_DEV AoDev)
{
    SAMPLE_AO_S* pstAoCtl = NULL;

    pstAoCtl = &gs_stSampleAo[AoCardId * AO_MAX_DEVICE_NUM + AoDev];
    if (pstAoCtl->bPauseCtrlStart) {
        pstAoCtl->bPauseCtrlStart = ES_FALSE;
        pthread_cancel(pstAoCtl->stAoPausePid);
        pthread_join(pstAoCtl->stAoPausePid, 0);
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Ao bind Adec
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_AoBindAdec(AUDIO_CARD AoCardId, AUDIO_DEV AoDev, ADEC_CHN AdChn)
{
    CHN_S stSrcChn, stDestChn;

    stSrcChn.modId = ES_ID_ADEC;
    stSrcChn.devId = 0;
    stSrcChn.chnId = AdChn;
    stDestChn.modId = ES_ID_AO;
    stDestChn.devId = AoDev;
    stDestChn.chnId = AoCardId;
    stSrcChn.nId = 0;
    stDestChn.nId = 0;

    return ES_SYS_Bind(&stSrcChn, &stDestChn);
}

/******************************************************************************
 * function : Ao unbind Adec
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_AoUnbindAdec(AUDIO_CARD AoCardId, AUDIO_DEV AoDev, ADEC_CHN AdChn)
{
    CHN_S stSrcChn, stDestChn;

    stSrcChn.modId = ES_ID_ADEC;
    stSrcChn.chnId = AdChn;
    stSrcChn.devId = 0;
    stDestChn.modId = ES_ID_AO;
    stDestChn.devId = AoDev;
    stDestChn.chnId = AoCardId;
    stSrcChn.nId = 0;
    stDestChn.nId = 0;

    return ES_SYS_UnBind(&stSrcChn, &stDestChn);
}

/******************************************************************************
 * function : Ao bind Ai
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_AoBindAi(AUDIO_CARD AiCard, AUDIO_DEV AiDev, AUDIO_CARD AoCard, AUDIO_DEV AoDev)
{
    CHN_S stSrcChn, stDestChn;

    stSrcChn.modId = ES_ID_AI;
    stSrcChn.chnId = AiCard;
    stSrcChn.devId = AiDev;
    stDestChn.modId = ES_ID_AO;
    stDestChn.devId = AoDev;
    stDestChn.chnId = AoCard;
    stSrcChn.nId = 0;
    stDestChn.nId = 0;

    return ES_SYS_Bind(&stSrcChn, &stDestChn);
}

/******************************************************************************
 * function : Ao unbind Ai
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_AoUnbindAi(AUDIO_CARD AiCard, AUDIO_DEV AiDev, AUDIO_CARD AoCard, AUDIO_DEV AoDev)
{
    CHN_S stSrcChn, stDestChn;

    stSrcChn.modId = ES_ID_AI;
    stSrcChn.chnId = AiCard;
    stSrcChn.devId = AiDev;
    stDestChn.modId = ES_ID_AO;
    stDestChn.devId = AoDev;
    stDestChn.chnId = AoCard;
    stSrcChn.nId = 0;
    stDestChn.nId = 0;

    return ES_SYS_UnBind(&stSrcChn, &stDestChn);
}

/******************************************************************************
 * function : Aenc bind Ai
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_AencBindAi(AUDIO_CARD AiCard, AUDIO_DEV AiDev, AENC_CHN AeChn)
{
    CHN_S stSrcChn, stDestChn;

    stSrcChn.modId = ES_ID_AI;
    stSrcChn.devId = AiDev;
    stSrcChn.chnId = AiCard;
    stDestChn.modId = ES_ID_AENC;
    stDestChn.devId = 0;
    stDestChn.chnId = AeChn;
    stSrcChn.nId = 0;
    stDestChn.nId = 0;

    return ES_SYS_Bind(&stSrcChn, &stDestChn);
}

/******************************************************************************
 * function : Aenc unbind Ai
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_AencUnbindAi(AUDIO_CARD AiCard, AUDIO_DEV AiDev, AENC_CHN AeChn)
{
    CHN_S stSrcChn, stDestChn;

    stSrcChn.modId = ES_ID_AI;
    stSrcChn.devId = AiDev;
    stSrcChn.chnId = AiCard;
    stDestChn.modId = ES_ID_AENC;
    stDestChn.devId = 0;
    stDestChn.chnId = AeChn;
    stSrcChn.nId = 0;
    stDestChn.nId = 0;

    return ES_SYS_UnBind(&stSrcChn, &stDestChn);
}

/******************************************************************************
 * function : Start Ai
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_StartAi(AUDIO_CARD AiCard, AUDIO_DEV AiDevId, AIO_ATTR_S* pstAioAttr,
                                 AUDIO_SAMPLE_RATE_E enOutSampleRate, ES_BOOL bResampleEn, ES_VOID* pstAiVqeAttr,
                                 ES_U32 u32AiVqeType)
{
    ES_S32 s32Ret;

    s32Ret = ES_AI_SetPubAttr(AiCard, AiDevId, pstAioAttr);
    if (s32Ret) {
        printf("%s: ES_AI_SetPubAttr(%d,%d) failed with %#x\n", __FUNCTION__, AiCard, AiDevId, s32Ret);
        return s32Ret;
    }

    s32Ret = ES_AI_Enable(AiCard, AiDevId);
    if (s32Ret) {
        printf("%s: ES_AI_Enable(%d) failed with %#x\n", __FUNCTION__, AiDevId, s32Ret);
        return s32Ret;
    }

    s32Ret = ES_AI_EnableChn(AiCard, AiDevId);
    if (s32Ret) {
        printf("%s: ES_AI_EnableChn(%d,%d) failed with %#x\n", __FUNCTION__, AiCard, AiDevId, s32Ret);
        return s32Ret;
    }

    if (ES_TRUE == bResampleEn) {
        s32Ret = ES_AI_EnableReSmp(AiCard, AiDevId, enOutSampleRate);
        if (s32Ret) {
            printf("%s: ES_AI_EnableReSmp(%d,%d) failed with %#x\n", __FUNCTION__, AiCard, AiDevId, s32Ret);
            return s32Ret;
        }
    }

    if (NULL != pstAiVqeAttr) {
        ES_BOOL bAiVqe = ES_TRUE;
        if (u32AiVqeType) {
            s32Ret = ES_AI_SetVqeAttr(AiCard, AiDevId, (AI_VQE_CONFIG_S*)pstAiVqeAttr);
            if (s32Ret) {
                printf("%s: SetAiVqe%d(%d,%d) failed with %#x\n", __FUNCTION__, u32AiVqeType, AiCard, AiDevId, s32Ret);
                return s32Ret;
            }
        }

        if (bAiVqe) {
            s32Ret = ES_AI_EnableVqe(AiCard, AiDevId);
            if (s32Ret) {
                printf("%s: ES_AI_EnableVqe(%d,%d) failed with %#x\n", __FUNCTION__, AiCard, AiDevId, s32Ret);
                return s32Ret;
            }
        }
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Stop Ai
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_StopAi(AUDIO_CARD AiCard, AUDIO_DEV AiDevId, ES_BOOL bResampleEn, ES_BOOL bVqeEn)
{
    ES_S32 s32Ret;

    if (ES_TRUE == bResampleEn) {
        s32Ret = ES_AI_DisableReSmp(AiCard, AiDevId);
        if (ES_SUCCESS != s32Ret) {
            printf("[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, "failed");
            return s32Ret;
        }
    }

    if (ES_TRUE == bVqeEn) {
        s32Ret = ES_AI_DisableVqe(AiCard, AiDevId);
        if (ES_SUCCESS != s32Ret) {
            printf("[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, "failed");
            return s32Ret;
        }
    }

    s32Ret = ES_AI_DisableChn(AiCard, AiDevId);
    if (ES_SUCCESS != s32Ret) {
        printf("[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, "failed");
        return s32Ret;
    }

    s32Ret = ES_AI_Disable(AiCard, AiDevId);
    if (ES_SUCCESS != s32Ret) {
        printf("[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, "failed");
        return s32Ret;
    }

    return ES_SUCCESS;
}

#ifdef ES_ACODEC_TYPE_HDMI
ES_S32 SAMPLE_COMM_AUDIO_StartHdmi(AIO_ATTR_S* pstAioAttr)
{
    ES_S32 s32Ret;
    ES_HDMI_ATTR_S stHdmiAttr;
    ES_HDMI_ID_E enHdmi = ES_HDMI_ID_0;
    VO_PUB_ATTR_S stPubAttr;
    VO_DEV VoDev = 0;

    stPubAttr.u32BgColor = 0x000000ff;
    stPubAttr.enIntfType = VO_INTF_HDMI;
    stPubAttr.enIntfSync = VO_OUTPUT_1080P30;

    if (ES_SUCCESS != SAMPLE_COMM_VO_StartDev(VoDev, &stPubAttr)) {
        printf("[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, "failed");
        return ES_FAILURE;
    }

    s32Ret = SAMPLE_COMM_VO_HdmiStart(stPubAttr.enIntfSync);
    if (ES_SUCCESS != s32Ret) {
        printf("[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, "failed");
        return ES_FAILURE;
    }

    s32Ret = ES_HDMI_Stop(enHdmi);
    if (ES_SUCCESS != s32Ret) {
        printf("[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, "failed");
        return ES_FAILURE;
    }

    s32Ret = ES_HDMI_GetAttr(enHdmi, &stHdmiAttr);
    if (ES_SUCCESS != s32Ret) {
        printf("[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, "failed");
        return ES_FAILURE;
    }

    stHdmiAttr.bEnableAudio = ES_TRUE;  // if enable audio
    stHdmiAttr.enVidInMode = ES_HDMI_VIDEO_MODE_YCBCR444;
    stHdmiAttr.enVidOutMode = ES_HDMI_VIDEO_MODE_YCBCR444;
    stHdmiAttr.enSoundIntf =
        ES_HDMI_SND_INTERFACE_I2S;  // source of HDMI audio, ES_HDMI_SND_INTERFACE_I2S suggested.the parameter must be
                                    // consistent with the input of AO
    stHdmiAttr.enSampleRate =
        pstAioAttr->enSamplerate;  // sampling rate of PCM audio,the parameter must be consistent with the input of AO
    stHdmiAttr.u8DownSampleParm = ES_FALSE;  // parameter of downsampling  rate of PCM audio, default :0

    stHdmiAttr.enBitDepth =
        8 * (pstAioAttr->enBitwidth +
             1);  // bitwidth of audio,default :16,the parameter must be consistent with the config of AO
    stHdmiAttr.u8I2SCtlVbit = 0;  // reserved, should be 0, I2S control (0x7A:0x1D) */

    stHdmiAttr.bEnableAviInfoFrame = ES_TRUE;  // if enable  AVI InfoFrame
    stHdmiAttr.bEnableAudInfoFrame = ES_TRUE;  // if enable AUDIO InfoFrame

    s32Ret = ES_HDMI_SetAttr(enHdmi, &stHdmiAttr);
    if (ES_SUCCESS != s32Ret) {
        printf("[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, "failed");
        return ES_FAILURE;
    }

    s32Ret = ES_HDMI_Start(enHdmi);
    if (ES_SUCCESS != s32Ret) {
        printf("[Func]:%s [Line]:%d [Info]:%s\n", __FUNCTION__, __LINE__, "failed");
        return ES_FAILURE;
    }

    return ES_SUCCESS;
}

ES_S32 SAMPLE_COMM_AUDIO_StopHdmi(ES_VOID)
{
    ES_S32 s32Ret;
    VO_DEV VoDev = 0;

    s32Ret = SAMPLE_COMM_VO_HdmiStop();
    s32Ret |= ES_VO_Disable(VoDev);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_VO_Disable failed with %#x!\n", __FUNCTION__, s32Ret);
        return ES_FAILURE;
    }

    return s32Ret;
}
#endif

/******************************************************************************
 * function : Start Ao with VQE
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_StartAoWithVQE(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, ES_S32 s32AoChnCnt,
                                        AIO_ATTR_S* pstAioAttr, AUDIO_SAMPLE_RATE_E enInSampleRate, ES_BOOL bResampleEn,
                                        AO_VQE_CONFIG_S* pstAoVqeAttr)
{
    ES_S32 s32Ret;

    s32Ret = ES_AO_SetPubAttr(AoCardId, AoDevId, pstAioAttr);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AO_SetPubAttr(%d) failed with %#x!\n", __FUNCTION__, AoDevId, s32Ret);
        return ES_FAILURE;
    }

    s32Ret = ES_AO_Enable(AoCardId, AoDevId);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AO_Enable(%d) failed with %#x!\n", __FUNCTION__, AoDevId, s32Ret);
        return ES_FAILURE;
    }

    s32Ret = ES_AO_EnableChn(AoCardId, AoDevId);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AO_EnableChn(%d,%d) failed with %#x!\n", __FUNCTION__, AoCardId, AoDevId, s32Ret);
        return ES_FAILURE;
    }
    printf("resample: %d\n", bResampleEn);
    if (ES_TRUE == bResampleEn) {
        s32Ret = ES_AO_DisableReSmp(AoCardId, AoDevId);
        s32Ret |= ES_AO_EnableReSmp(AoCardId, AoDevId, enInSampleRate);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AO_EnableReSmp(%d,%d) failed with %#x!\n", __FUNCTION__, AoCardId, AoDevId, s32Ret);
            return ES_FAILURE;
        }
    }

    if (ES_NULL != pstAoVqeAttr) {
        s32Ret = ES_AO_SetVqeAttr(AoCardId, AoDevId, pstAoVqeAttr);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AO_SetVqeAttr(%d, %d) failed with %#x!\n", __FUNCTION__, AoCardId, AoDevId, s32Ret);
            return ES_FAILURE;
        }
        s32Ret = ES_AO_EnableVqe(AoCardId, AoDevId);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AO_EnableVqe(%d,%d) failed with %#x!\n", __FUNCTION__, AoCardId, AoDevId, s32Ret);
            return ES_FAILURE;
        }
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Start Ao
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_StartAo(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, ES_S32 s32AoChnCnt, AIO_ATTR_S* pstAioAttr,
                                 AUDIO_SAMPLE_RATE_E enInSampleRate, ES_BOOL bResampleEn)
{
    ES_S32 s32Ret;

    s32Ret = ES_AO_SetPubAttr(AoCardId, AoDevId, pstAioAttr);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AO_SetPubAttr(%d,%d) failed with %#x!\n", __FUNCTION__, AoCardId, AoDevId, s32Ret);
        return ES_FAILURE;
    }

    s32Ret = ES_AO_Enable(AoCardId, AoDevId);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AO_Enable(%d,%d) failed with %#x!\n", __FUNCTION__, AoCardId, AoDevId, s32Ret);
        return ES_FAILURE;
    }

    s32Ret = ES_AO_EnableChn(AoCardId, AoDevId);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AO_EnableChn(%d,%d) failed with %#x!\n", __FUNCTION__, AoCardId, AoDevId, s32Ret);
        return ES_FAILURE;
    }

    printf("ES_AO_EnableChn: (%d,%d) success\n", AoCardId, AoDevId);

    if (ES_TRUE == bResampleEn) {
        s32Ret = ES_AO_DisableReSmp(AoCardId, AoDevId);
        s32Ret |= ES_AO_EnableReSmp(AoCardId, AoDevId, enInSampleRate);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AO_EnableReSmp(%d,%d) failed with %#x!\n", __FUNCTION__, AoCardId, AoDevId, s32Ret);
            return ES_FAILURE;
        }
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Stop Ao
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_StopAo(AUDIO_CARD AoCardId, AUDIO_DEV AoDevId, ES_S32 s32AoChnCnt, ES_BOOL bResampleEn)
{
    ES_S32 s32Ret;

    if (ES_TRUE == bResampleEn) {
        s32Ret = ES_AO_DisableReSmp(AoCardId, AoDevId);
        if (ES_SUCCESS != s32Ret) {
            printf("%s: ES_AO_DisableReSmp failed with %#x!\n", __FUNCTION__, s32Ret);
            return s32Ret;
        }
    }

    s32Ret = ES_AO_DisableChn(AoCardId, AoDevId);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AO_DisableChn failed with %#x!\n", __FUNCTION__, s32Ret);
        return s32Ret;
    }

    s32Ret = ES_AO_Disable(AoCardId, AoDevId);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AO_Disable failed with %#x!\n", __FUNCTION__, s32Ret);
        return s32Ret;
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Start Aenc
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_StartAenc(ES_S32 s32AencChn, AIO_ATTR_S* pstAioAttr, PAYLOAD_TYPE_E enType)
{
    AENC_CHN AeChn;
    ES_S32 s32Ret;
    AENC_CHN_ATTR_S stAencAttr;
    AENC_ATTR_G711_S stAencG711;
    AENC_ATTR_G722_S stAencG722;
    AENC_ATTR_G726_S stAencG726;
    AENC_ATTR_AAC_S stAencAac;
    AENC_ATTR_AMR_S stAencAmr;

    /* set AENC chn attr */

    stAencAttr.Type = enType;
    stAencAttr.BufSize = 30;
    stAencAttr.PtNumPerFrm = pstAioAttr->PtNumPerFrm;

    if (PT_G722 == stAencAttr.Type) {
        stAencAttr.Value = &stAencG722;
        stAencG722.BitRate = 64000;
    } else if (PT_G711A == stAencAttr.Type || PT_G711U == stAencAttr.Type) {
        stAencAttr.Value = &stAencG711;
        stAencG711.IsULaw = 0;
    } else if (PT_G726 == stAencAttr.Type) {
        stAencAttr.Value = &stAencG726;
        stAencG726.BitRate = 64000;
    } else if (PT_AMR == stAencAttr.Type) {
        stAencAttr.Value = &stAencAmr;
        stAencAmr.BitRate = 64000;
        stAencAmr.Dtx = 0;
        stAencAmr.IsWb = 0;
    } else if (PT_AAC == stAencAttr.Type) {
        stAencAttr.Value = &stAencAac;
        stAencAac.BitRate = 64000;
        stAencAac.Aot = 2;
        stAencAac.Sbr = 1;
        stAencAac.Channels = soundModeToChannels[pstAioAttr->Soundmode];
        stAencAac.Vbr = 0;
        stAencAac.SampleRate = (int32_t)pstAioAttr->Samplerate;
        stAencAac.Afterburner = 1;
    } else if (PT_ADPCMA == stAencAttr.Type) {
        stAencAttr.Value = NULL;
    } else {
        printf("%s: invalid aenc payload type:%d\n", __FUNCTION__, stAencAttr.Type);
        return ES_FAILURE;
    }

    AeChn = s32AencChn;

    /* create aenc chn*/
    s32Ret = ES_AENC_CreateChn(AeChn, &stAencAttr);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AENC_CreateChn(%d) failed with %#x!\n", __FUNCTION__, AeChn, s32Ret);
        return s32Ret;
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Stop Aenc
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_StopAenc(AENC_CHN AeChn)
{
    ES_S32 s32Ret;

    s32Ret = ES_AENC_DestroyChn(AeChn);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_AENC_DestroyChn(%d) failed with %#x!\n", __FUNCTION__, AeChn, s32Ret);
        return s32Ret;
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Destroy the all thread
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_DestroyAllTrd(void)
{
    ES_U32 u32DevId, u32ChnId;
    ES_U32 u32CardId;

    for (u32CardId = 0; u32CardId < AI_MAX_CARD_NUM; u32CardId++) {
        for (u32DevId = 0; u32DevId < AI_MAX_DEVICE_NUM; u32DevId++) {
            if (ES_SUCCESS != SAMPLE_COMM_AUDIO_DestroyTrdAi(u32CardId, u32DevId)) {
                printf("%s: SAMPLE_COMM_AUDIO_DestroyTrdAi(%d,%d) failed!\n", __FUNCTION__, u32CardId, u32DevId);
                return ES_FAILURE;
            }
            if (ES_SUCCESS != SAMPLE_COMM_AUDIO_DestroyTrdAiVolCtrl(u32CardId, u32DevId)) {
                printf("%s: SAMPLE_COMM_AUDIO_DestroyTrdAiVolCtrl(%d,%d) failed!\n", __FUNCTION__, u32CardId, u32DevId);
                return ES_FAILURE;
            }
        }
    }

    for (u32ChnId = 0; u32ChnId < AENC_MAX_CHN_NUM; u32ChnId++) {
        if (ES_SUCCESS != SAMPLE_COMM_AUDIO_DestroyTrdAencAdec(u32ChnId)) {
            printf("%s: SAMPLE_COMM_AUDIO_DestroyTrdAencAdec(%d) failed!\n", __FUNCTION__, u32ChnId);
            return ES_FAILURE;
        }
    }

    for (u32ChnId = 0; u32ChnId < ADEC_MAX_CHN_NUM; u32ChnId++) {
        if (ES_SUCCESS != SAMPLE_COMM_AUDIO_DestroyTrdFileAdec(u32ChnId)) {
            printf("%s: SAMPLE_COMM_AUDIO_DestroyTrdFileAdec(%d) failed!\n", __FUNCTION__, u32ChnId);
            return ES_FAILURE;
        }
    }

    for (u32CardId = 0; u32CardId < AO_MAX_CARD_NUM; u32CardId++) {
        for (u32DevId = 0; u32DevId < AO_MAX_DEVICE_NUM; u32DevId++) {
            if (ES_SUCCESS != SAMPLE_COMM_AUDIO_DestroyTrdAo(u32CardId, u32DevId)) {
                printf("%s: SAMPLE_COMM_AUDIO_DestroyTrdAo(%d, %d,%d) failed!\n", __FUNCTION__, u32CardId, u32DevId,
                       u32ChnId);
                return ES_FAILURE;
            }
            if (ES_SUCCESS != SAMPLE_COMM_AUDIO_DestroyTrdAoVolCtrl(u32CardId, u32DevId)) {
                printf("%s: SAMPLE_COMM_AUDIO_DestroyTrdAoVolCtrl(%d,%d) failed!\n", __FUNCTION__, u32CardId, u32DevId);
                return ES_FAILURE;
            }
        }
    }

    return ES_SUCCESS;
}

/******************************************************************************
 * function : Start Adec
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_StartAdec(ADEC_CHN AdChn, PAYLOAD_TYPE_E enType)
{
    ES_S32 s32Ret;
    ADEC_CHN_ATTR_S stAdecAttr;
    ADEC_ATTR_G711_S stAdecG711;
    ADEC_ATTR_AAC_S stAdecAac;
    ADEC_ATTR_AMR_S stAdecAmr;

    stAdecAttr.Type = enType;
    stAdecAttr.BufSize = 30;
    stAdecAttr.Mode = ADEC_MODE_STREAM;

    if (PT_G722 == stAdecAttr.Type) {
        stAdecAttr.Value = NULL;
    } else if (PT_G711A == stAdecAttr.Type || PT_G711U == stAdecAttr.Type) {
        stAdecAttr.Value = &stAdecG711;
        if (PT_G711A == stAdecAttr.Type) {
            stAdecG711.IsULaw = 0;
        } else if (PT_G711U == stAdecAttr.Type) {
            stAdecG711.IsULaw = 1;
        }
    } else if (PT_G726 == stAdecAttr.Type) {
        stAdecAttr.Value = NULL;
    } else if (PT_AAC == stAdecAttr.Type) {
        stAdecAttr.Value = &stAdecAac;
        stAdecAac.OutputFormat = 1;
        stAdecAac.TransType = ADEC_AAC_TRANS_TYPE_ADTS;
        stAdecAac.Profile = ADEC_AAC_TYPE_AACLC;
    } else if (PT_AMR == stAdecAttr.Type) {
        stAdecAttr.Value = &stAdecAmr;
        stAdecAmr.IsWb = 0;
    } else if (PT_ADPCMA == stAdecAttr.Type) {
        stAdecAttr.Value = NULL;
    } else {
        printf("%s: invalid aenc payload type:%d\n", __FUNCTION__, stAdecAttr.Type);
        return ES_FAILURE;
    }

    /* create adec chn*/
    s32Ret = ES_ADEC_CreateChn(AdChn, &stAdecAttr);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_ADEC_CreateChn(%d) failed with %#x!\n", __FUNCTION__, AdChn, s32Ret);
        return s32Ret;
    }
    return 0;
}

/******************************************************************************
 * function : Stop Adec
 ******************************************************************************/
ES_S32 SAMPLE_COMM_AUDIO_StopAdec(ADEC_CHN AdChn)
{
    ES_S32 s32Ret;

    s32Ret = ES_ADEC_DestroyChn(AdChn);
    if (ES_SUCCESS != s32Ret) {
        printf("%s: ES_ADEC_DestroyChn(%d) failed with %#x!\n", __FUNCTION__, AdChn, s32Ret);
        return s32Ret;
    }

    return ES_SUCCESS;
}

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* End of #ifdef __cplusplus */
