// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
#include "utils.h"
#include "es_audio.h"

#define ES_AUDIO_MAJOR_VERSION 0
#define ES_AUDIO_MINOR_VERSION 0
#define ES_AUDIO_PATCH_VERSION 1
#define ID_RIFF 0x46464952
#define ID_WAVE 0x45564157
#define ID_FMT 0x20746d66
#define ID_DATA 0x61746164

struct riff_wave_header {
    uint32_t riff_id;
    uint32_t riff_sz;
    uint32_t wave_id;
};
struct chunk_header {
    uint32_t id;
    uint32_t sz;
};

ES_S32 parseWav(FILE *file, struct chunk_fmt *fmt)
{
    struct chunk_header chunk_header;
    struct riff_wave_header wave_header;
    int more_chunks = 1;

    size_t ret = fread(&wave_header, sizeof(wave_header), 1, file);
    if (ret <= 0 || (wave_header.riff_id != ID_RIFF) || (wave_header.wave_id != ID_WAVE)) {
        return ES_FAILURE;
    }
    do {
        ret = fread(&chunk_header, sizeof(chunk_header), 1, file);
        switch (chunk_header.id) {
        case ID_FMT:
            ret = fread(fmt, sizeof(*fmt), 1, file);
            if (chunk_header.sz > sizeof(*fmt)) fseek(file, chunk_header.sz - sizeof(*fmt), SEEK_CUR);
            break;
        case ID_DATA:
            more_chunks = 0;
            break;
        default:
            fseek(file, chunk_header.sz, SEEK_CUR);
        }
    } while (more_chunks);

    return ES_SUCCESS;
}

AUDIO_BIT_WIDTH_E convertBitsToBitWidth(ES_S32 bits)
{
    switch (bits) {
    case 8:
        return AUDIO_BIT_WIDTH_8;
        break;
    case 16:
        return AUDIO_BIT_WIDTH_16;
        break;
    case 24:
        return AUDIO_BIT_WIDTH_24;
        break;
    case 32:
        return AUDIO_BIT_WIDTH_32;
        break;
    default:
        return AUDIO_BIT_WIDTH_BUTT;
    }
}

AUDIO_SOUND_MODE_E convertChnsToSoundMode(ES_S32 chns)
{
    switch (chns) {
    case 1:
        return AUDIO_SOUND_MODE_MONO;
    case 2:
        return AUDIO_SOUND_MODE_STEREO;
    default:
        return AUDIO_SOUND_MODE_BUTT;
    }
}

ES_S32 ES_Audio_GetVersion(ES_U64 *version)
{
    ES_SDK_VERSION_U *v = (ES_SDK_VERSION_U *)version;

    v->VERSION_NUM_S.major = ES_AUDIO_MAJOR_VERSION;
    v->VERSION_NUM_S.minor = ES_AUDIO_MINOR_VERSION;
    v->VERSION_NUM_S.patch = ES_AUDIO_PATCH_VERSION;

    return ES_SUCCESS;
}

ES_S32 ES_Audio_GetVersionString(ES_CHAR *version, ES_U32 maxSize)
{
    ES_SDK_VERSION_U v;

    ES_Audio_GetVersion(&v.version);

    ES_S32 ret =
        snprintf(version, maxSize, "v%u.%hu.%hu", v.VERSION_NUM_S.major, v.VERSION_NUM_S.minor, v.VERSION_NUM_S.patch);
    if (ret < 0 || ret + 1 > maxSize) {
        return ES_FAILURE;
    }

    return ES_SUCCESS;
}
